@@ -9,10 +9,11 @@
 LICENSE END */
 use crate::diagnostics::{BuildDiagnostics, SourceLocation, Spanned};
 use crate::langtype::{BuiltinElement, EnumerationValue, Type};
+use crate::layout::Orientation;
 use crate::object_tree::*;
 use crate::parser::{NodeOrToken, SyntaxNode};
 use core::cell::RefCell;
-use std::collections::HashMap;
+use std::collections::{BTreeMap, HashMap};
 use std::rc::{Rc, Weak};
 
 // FIXME remove the pub
@@ -27,6 +28,7 @@ pub enum BuiltinFunction {
     Round,
     Ceil,
     Floor,
+    Abs,
     Sqrt,
     Cos,
     Sin,
@@ -42,8 +44,9 @@ pub enum BuiltinFunction {
     StringIsFloat,
     ColorBrighter,
     ColorDarker,
+    ImageSize,
     Rgb,
-    ImplicitItemSize,
+    ImplicitLayoutInfo(Orientation),
     RegisterCustomFontByPath,
     RegisterCustomFontByMemory,
 }
@@ -55,6 +58,7 @@ pub enum BuiltinMacroFunction {
     Max,
     CubicBezier,
     Rgb,
+    Debug,
 }
 
 impl BuiltinFunction {
@@ -74,7 +78,7 @@ impl BuiltinFunction {
             BuiltinFunction::Round | BuiltinFunction::Ceil | BuiltinFunction::Floor => {
                 Type::Function { return_type: Box::new(Type::Int32), args: vec![Type::Float32] }
             }
-            BuiltinFunction::Sqrt => {
+            BuiltinFunction::Sqrt | BuiltinFunction::Abs => {
                 Type::Function { return_type: Box::new(Type::Float32), args: vec![Type::Float32] }
             }
             BuiltinFunction::Cos | BuiltinFunction::Sin | BuiltinFunction::Tan => {
@@ -97,18 +101,8 @@ impl BuiltinFunction {
             BuiltinFunction::StringIsFloat => {
                 Type::Function { return_type: Box::new(Type::Bool), args: vec![Type::String] }
             }
-            BuiltinFunction::ImplicitItemSize => Type::Function {
-                return_type: Box::new(Type::Struct {
-                    fields: [
-                        ("width".to_string(), Type::LogicalLength),
-                        ("height".to_string(), Type::LogicalLength),
-                    ]
-                    .iter()
-                    .cloned()
-                    .collect(),
-                    name: Some("Size".to_string()),
-                    node: None,
-                }),
+            BuiltinFunction::ImplicitLayoutInfo(_) => Type::Function {
+                return_type: Box::new(crate::layout::layout_info_type()),
                 args: vec![Type::ElementReference],
             },
             BuiltinFunction::ColorBrighter => Type::Function {
@@ -119,6 +113,18 @@ impl BuiltinFunction {
                 return_type: Box::new(Type::Color),
                 args: vec![Type::Color, Type::Float32],
             },
+            BuiltinFunction::ImageSize => Type::Function {
+                return_type: Box::new(Type::Struct {
+                    fields: std::array::IntoIter::new([
+                        ("width".to_string(), Type::Int32),
+                        ("height".to_string(), Type::Int32),
+                    ])
+                    .collect(),
+                    name: Some("Size".to_string()),
+                    node: None,
+                }),
+                args: vec![Type::Image],
+            },
             BuiltinFunction::Rgb => Type::Function {
                 return_type: Box::new(Type::Color),
                 args: vec![Type::Int32, Type::Int32, Type::Int32, Type::Float32],
@@ -131,6 +137,36 @@ impl BuiltinFunction {
             }
         }
     }
+
+    /// It is pure if the return value only depends on its argument and has no side effect
+    fn is_pure(&self) -> bool {
+        match self {
+            BuiltinFunction::GetWindowScaleFactor => false,
+            // Even if it is not pure, we optimize it away anyway
+            BuiltinFunction::Debug => true,
+            BuiltinFunction::Mod
+            | BuiltinFunction::Round
+            | BuiltinFunction::Ceil
+            | BuiltinFunction::Floor
+            | BuiltinFunction::Abs
+            | BuiltinFunction::Sqrt
+            | BuiltinFunction::Cos
+            | BuiltinFunction::Sin
+            | BuiltinFunction::Tan
+            | BuiltinFunction::ACos
+            | BuiltinFunction::ASin
+            | BuiltinFunction::ATan => true,
+            BuiltinFunction::SetFocusItem => false,
+            BuiltinFunction::ShowPopupWindow => false,
+            BuiltinFunction::StringToFloat | BuiltinFunction::StringIsFloat => true,
+            BuiltinFunction::ColorBrighter | BuiltinFunction::ColorDarker => true,
+            BuiltinFunction::ImageSize => true,
+            BuiltinFunction::Rgb => true,
+            BuiltinFunction::ImplicitLayoutInfo(_) => false,
+            BuiltinFunction::RegisterCustomFontByPath
+            | BuiltinFunction::RegisterCustomFontByMemory => false,
+        }
+    }
 }
 
 #[derive(Debug, Clone, Eq, PartialEq)]
@@ -207,12 +243,12 @@ declare_units! {
     Px = "px" -> LogicalLength,
     /// Centimeters
     Cm = "cm" -> LogicalLength * 37.8,
-    /// Milimeters
+    /// Millimeters
     Mm = "mm" -> LogicalLength * 3.78,
     /// inches
     In = "in" -> LogicalLength * 96,
     /// Points
-    Pt = "pt" -> LogicalLength * 96/72,
+    Pt = "pt" -> LogicalLength * 96./72.,
 
     // durations
 
@@ -226,9 +262,9 @@ declare_units! {
     /// Degree
     Deg = "deg" -> Angle,
     /// Gradians
-    Grad = "grad" -> Angle * 400/360,
+    Grad = "grad" -> Angle * 400./360.,
     /// Turns
-    Turn = "turn" -> Angle * 1/360,
+    Turn = "turn" -> Angle * 1./360.,
     /// Radians
     Rad = "rad" -> Angle * std::f32::consts::TAU/360.,
 }
@@ -269,13 +305,13 @@ pub enum Expression {
     PropertyReference(NamedReference),
 
     /// Reference to a function built into the run-time, implemented natively
-    BuiltinFunctionReference(BuiltinFunction),
+    BuiltinFunctionReference(BuiltinFunction, Option<SourceLocation>),
 
     /// A MemberFunction expression exists only for a short time, for example for `item.focus()` to be translated to
     /// a regular FunctionCall expression where the base becomes the first argument.
     MemberFunction {
         base: Box<Expression>,
-        base_node: NodeOrToken,
+        base_node: Option<NodeOrToken>,
         member: Box<Expression>,
     },
 
@@ -345,7 +381,7 @@ pub enum Expression {
         source_location: Option<SourceLocation>,
     },
 
-    /// A SelfAssignment or an Assignment.  When op is '=' this is a signel assignment.
+    /// A SelfAssignment or an Assignment.  When op is '=' this is a signal assignment.
     SelfAssignment {
         lhs: Box<Expression>,
         rhs: Box<Expression>,
@@ -398,6 +434,18 @@ pub enum Expression {
     EnumerationValue(EnumerationValue),
 
     ReturnStatement(Option<Box<Expression>>),
+
+    LayoutCacheAccess {
+        layout_cache_prop: NamedReference,
+        index: usize,
+        /// When set, this is the index within a repeater, and the index is then the location of another offset.
+        /// So this looks like `layout_cache_prop[layout_cache_prop[index] + repeater_index]`
+        repeater_index: Option<Box<Expression>>,
+    },
+    /// Compute the LayoutInfo for the given layout.
+    /// The orientation is the orientation of the cache, not the orientation of the layout
+    ComputeLayoutInfo(crate::layout::Layout, crate::layout::Orientation),
+    SolveLayout(crate::layout::Layout, crate::layout::Orientation),
 }
 
 impl Default for Expression {
@@ -418,7 +466,7 @@ impl Expression {
             Expression::TwoWayBinding(nr, _) => nr.ty(),
             Expression::CallbackReference(nr) => nr.ty(),
             Expression::PropertyReference(nr) => nr.ty(),
-            Expression::BuiltinFunctionReference(funcref) => funcref.ty(),
+            Expression::BuiltinFunctionReference(funcref, _) => funcref.ty(),
             Expression::MemberFunction { member, .. } => member.ty(),
             Expression::BuiltinMacroReference { .. } => Type::Invalid, // We don't know the type
             Expression::ElementReference(_) => Type::ElementReference,
@@ -530,6 +578,9 @@ impl Expression {
             Expression::EnumerationValue(value) => Type::Enumeration(value.enumeration.clone()),
             // invalid because the expression is unreachable
             Expression::ReturnStatement(_) => Type::Invalid,
+            Expression::LayoutCacheAccess { .. } => Type::LogicalLength,
+            Expression::ComputeLayoutInfo(..) => crate::layout::layout_info_type(),
+            Expression::SolveLayout(..) => Type::LayoutCache,
         }
     }
 
@@ -611,8 +662,13 @@ impl Expression {
             }
             Expression::EnumerationValue(_) => {}
             Expression::ReturnStatement(expr) => {
-                expr.as_deref().map(|expr| visitor(expr));
+                expr.as_deref().map(visitor);
             }
+            Expression::LayoutCacheAccess { repeater_index, .. } => {
+                repeater_index.as_deref().map(visitor);
+            }
+            Expression::ComputeLayoutInfo(..) => {}
+            Expression::SolveLayout(..) => {}
         }
     }
 
@@ -693,8 +749,13 @@ impl Expression {
             }
             Expression::EnumerationValue(_) => {}
             Expression::ReturnStatement(expr) => {
-                expr.as_deref_mut().map(|expr| visitor(expr));
+                expr.as_deref_mut().map(visitor);
             }
+            Expression::LayoutCacheAccess { repeater_index, .. } => {
+                repeater_index.as_deref_mut().map(visitor);
+            }
+            Expression::ComputeLayoutInfo(..) => {}
+            Expression::SolveLayout(..) => {}
         }
     }
 
@@ -708,26 +769,33 @@ impl Expression {
         match self {
             Expression::Invalid => true,
             Expression::Uncompiled(_) => false,
-            Expression::TwoWayBinding(..) => false,
+            Expression::TwoWayBinding(nr, expr) => {
+                nr.is_constant() && expr.as_ref().map_or(true, |e| e.is_constant())
+            }
             Expression::StringLiteral(_) => true,
             Expression::NumberLiteral(_, _) => true,
             Expression::BoolLiteral(_) => true,
             Expression::CallbackReference { .. } => false,
-            Expression::PropertyReference { .. } => false,
-            Expression::BuiltinFunctionReference { .. } => false,
+            Expression::PropertyReference(nr) => nr.is_constant(),
+            Expression::BuiltinFunctionReference(func, _) => func.is_pure(),
             Expression::MemberFunction { .. } => false,
             Expression::ElementReference(_) => false,
             Expression::RepeaterIndexReference { .. } => false,
             Expression::RepeaterModelReference { .. } => false,
             Expression::FunctionParameterReference { .. } => false,
-            Expression::BuiltinMacroReference { .. } => false,
+            Expression::BuiltinMacroReference { .. } => true,
             Expression::StructFieldAccess { base, .. } => base.is_constant(),
             Expression::Cast { from, .. } => from.is_constant(),
             Expression::CodeBlock(sub) => sub.len() == 1 && sub.first().unwrap().is_constant(),
-            Expression::FunctionCall { .. } => false,
+            Expression::FunctionCall { function, arguments, .. } => {
+                // Assume that constant function are, in fact, pure
+                function.is_constant() && arguments.iter().all(|a| a.is_constant())
+            }
             Expression::SelfAssignment { .. } => false,
             Expression::ImageReference { .. } => true,
-            Expression::Condition { .. } => false,
+            Expression::Condition { condition, false_expr, true_expr } => {
+                condition.is_constant() && false_expr.is_constant() && true_expr.is_constant()
+            }
             Expression::BinaryExpression { lhs, rhs, .. } => lhs.is_constant() && rhs.is_constant(),
             Expression::UnaryOp { sub, .. } => sub.is_constant(),
             Expression::Array { values, .. } => values.iter().all(Expression::is_constant),
@@ -742,6 +810,7 @@ impl Expression {
                 }
             }
             Expression::StoreLocalVariable { .. } => false,
+            // we should somehow find out if this is constant or not
             Expression::ReadLocalVariable { .. } => false,
             Expression::EasingCurve(_) => true,
             Expression::LinearGradient { angle, stops } => {
@@ -751,6 +820,10 @@ impl Expression {
             Expression::ReturnStatement(expr) => {
                 expr.as_ref().map_or(true, |expr| expr.is_constant())
             }
+            // TODO:  detect constant property within layouts
+            Expression::LayoutCacheAccess { .. } => false,
+            Expression::ComputeLayoutInfo(..) => false,
+            Expression::SolveLayout(..) => false,
         }
     }
 
@@ -846,6 +919,7 @@ impl Expression {
                                     rhs: Box::new(Expression::FunctionCall {
                                         function: Box::new(Expression::BuiltinFunctionReference(
                                             BuiltinFunction::GetWindowScaleFactor,
+                                            Some(node.to_source_location()),
                                         )),
                                         arguments: vec![],
                                         source_location: Some(node.to_source_location()),
@@ -878,7 +952,7 @@ impl Expression {
             }
         } else {
             let mut message = format!("Cannot convert {} to {}", ty, target_type);
-            // Explicit error message for unit cnversion
+            // Explicit error message for unit conversion
             if let Some(from_unit) = ty.default_unit() {
                 if matches!(&target_type, Type::Int32 | Type::Float32 | Type::String) {
                     message = format!(
@@ -895,7 +969,7 @@ impl Expression {
                         }
                     }
                     message = format!(
-                        "{}. Use an unit, or multiply by 1{} to convert explicitly.",
+                        "{}. Use an unit, or multiply by 1{} to convert explicitly",
                         message, to_unit
                     );
                 }
@@ -915,7 +989,10 @@ impl Expression {
             | Type::Callback { .. }
             | Type::Function { .. }
             | Type::Void
-            | Type::ElementReference => Expression::Invalid,
+            | Type::InferredProperty
+            | Type::InferredCallback
+            | Type::ElementReference
+            | Type::LayoutCache => Expression::Invalid,
             Type::Float32 => Expression::NumberLiteral(0., Unit::None),
             Type::Int32 => Expression::NumberLiteral(0., Unit::None),
             Type::String => Expression::StringLiteral(String::new()),
@@ -958,11 +1035,22 @@ impl Expression {
         }
     }
 
+    /// Try to mark this expression to a lvalue that can be assigned to.
+    ///
     /// Return true if the expression is a "lvalue" that can be used as the left hand side of a `=` or `+=` or similar
-    pub fn is_rw(&self) -> bool {
+    pub fn try_set_rw(&mut self) -> bool {
         match self {
-            Expression::PropertyReference(_) => true,
-            Expression::StructFieldAccess { base, .. } => base.is_rw(),
+            Expression::PropertyReference(nr) => {
+                nr.element()
+                    .borrow()
+                    .property_analysis
+                    .borrow_mut()
+                    .entry(nr.name().to_owned())
+                    .or_default()
+                    .is_set = true;
+                true
+            }
+            Expression::StructFieldAccess { base, .. } => base.try_set_rw(),
             Expression::RepeaterModelReference { .. } => true,
             _ => false,
         }
@@ -970,7 +1058,7 @@ impl Expression {
 }
 
 /// The expression in the Element::binding hash table
-#[derive(Default, Debug, Clone, derive_more::Deref, derive_more::DerefMut)]
+#[derive(Debug, Clone, derive_more::Deref, derive_more::DerefMut)]
 pub struct BindingExpression {
     #[deref]
     #[deref_mut]
@@ -979,12 +1067,17 @@ pub struct BindingExpression {
     pub span: Option<SourceLocation>,
     /// How deep is this binding declared in the hierarchy. When two binding are conflicting
     /// for the same priority (because of two way binding), the lower priority wins.
+    /// The priority starts at 1, and each level of inlining adds one to the priority.
+    /// 0 means the expression was added by some passes and it is not explicit in the source code
     pub priority: i32,
+
+    /// The analysis information. None before it is computed
+    pub analysis: RefCell<Option<BindingAnalysis>>,
 }
 
 impl std::convert::From<Expression> for BindingExpression {
     fn from(expression: Expression) -> Self {
-        Self { expression, span: None, priority: 0 }
+        Self { expression, span: None, priority: 0, analysis: Default::default() }
     }
 }
 
@@ -993,9 +1086,13 @@ impl BindingExpression {
         Self {
             expression: Expression::Uncompiled(node.clone()),
             span: Some(node.to_source_location()),
-            priority: 0,
+            priority: 1,
+            analysis: Default::default(),
         }
     }
+    pub fn new_with_span(expression: Expression, span: SourceLocation) -> Self {
+        Self { expression, span: Some(span), priority: 0, analysis: Default::default() }
+    }
 }
 
 impl Spanned for BindingExpression {
@@ -1007,6 +1104,15 @@ impl Spanned for BindingExpression {
     }
 }
 
+#[derive(Default, Debug, Clone)]
+pub struct BindingAnalysis {
+    /// true if that binding is part of a binding loop that already has been reported.
+    pub is_in_binding_loop: bool,
+
+    /// true if the binding is a constant value that can be set without creating a binding at runtime
+    pub is_const: bool,
+}
+
 pub type PathEvents = Vec<lyon_path::Event<lyon_path::math::Point, lyon_path::math::Point>>;
 
 #[derive(Debug, Clone)]
@@ -1018,14 +1124,14 @@ pub enum Path {
 #[derive(Debug, Clone)]
 pub struct PathElement {
     pub element_type: Rc<BuiltinElement>,
-    pub bindings: HashMap<String, BindingExpression>,
+    pub bindings: BTreeMap<String, BindingExpression>,
 }
 
 #[derive(Clone, Debug)]
 pub enum EasingCurve {
     Linear,
     CubicBezier(f32, f32, f32, f32),
-    // CubicBesizerNonConst([Box<Expression>; 4]),
+    // CubicBezierNonConst([Box<Expression>; 4]),
     // Custom(Box<dyn Fn(f32)->f32>),
 }
 
@@ -1035,7 +1141,7 @@ impl Default for EasingCurve {
     }
 }
 
-// The compiler generates ResourceReference::AbsolutePath for all references likg @image-url("foo.png")
+// The compiler generates ResourceReference::AbsolutePath for all references like @image-url("foo.png")
 // and the resource lowering path may change this to EmbeddedData if configured.
 #[derive(Clone, Debug)]
 pub enum ImageReference {
@@ -1044,7 +1150,7 @@ pub enum ImageReference {
     EmbeddedData(usize),
 }
 
-/// Print the expression as a .60 code (not nessecarily valid .60)
+/// Print the expression as a .60 code (not necessarily valid .60)
 pub fn pretty_print(f: &mut dyn std::fmt::Write, expression: &Expression) -> std::fmt::Result {
     match expression {
         Expression::Invalid => write!(f, "<invalid>"),
@@ -1062,7 +1168,7 @@ pub fn pretty_print(f: &mut dyn std::fmt::Write, expression: &Expression) -> std
         Expression::BoolLiteral(b) => write!(f, "{:?}", b),
         Expression::CallbackReference(a) => write!(f, "{:?}", a),
         Expression::PropertyReference(a) => write!(f, "{:?}", a),
-        Expression::BuiltinFunctionReference(a) => write!(f, "{:?}", a),
+        Expression::BuiltinFunctionReference(a, _) => write!(f, "{:?}", a),
         Expression::MemberFunction { base, base_node: _, member } => {
             pretty_print(f, base)?;
             write!(f, ".")?;
@@ -1176,31 +1282,16 @@ pub fn pretty_print(f: &mut dyn std::fmt::Write, expression: &Expression) -> std
             write!(f, "return ")?;
             e.as_ref().map(|e| pretty_print(f, e)).unwrap_or(Ok(()))
         }
+        Expression::LayoutCacheAccess { layout_cache_prop, index, repeater_index } => {
+            write!(
+                f,
+                "{:?}[{}{}]",
+                layout_cache_prop,
+                index,
+                if repeater_index.is_some() { " + $index" } else { "" }
+            )
+        }
+        Expression::ComputeLayoutInfo(..) => write!(f, "layout_info(..)"),
+        Expression::SolveLayout(..) => write!(f, "solve_layout(..)"),
     }
 }
-
-/// Generate an expression which is like `min(lhs, rhs)` if op is '<' or `max(lhs, rhs)` if op is '>'.
-/// counter is an unique id.
-/// The rhs and lhs of the expression must have the same numerical type
-pub fn min_max_expression(lhs: Expression, rhs: Expression, op: char) -> Expression {
-    let ty = lhs.ty();
-    static COUNTER: std::sync::atomic::AtomicUsize = std::sync::atomic::AtomicUsize::new(1);
-    let id = COUNTER.fetch_add(1, std::sync::atomic::Ordering::Relaxed);
-    let n1 = format!("minmax_lhs{}", id);
-    let n2 = format!("minmax_rhs{}", id);
-    let a1 = Box::new(Expression::ReadLocalVariable { name: n1.clone(), ty: ty.clone() });
-    let a2 = Box::new(Expression::ReadLocalVariable { name: n2.clone(), ty });
-    Expression::CodeBlock(vec![
-        Expression::StoreLocalVariable { name: n1, value: Box::new(lhs) },
-        Expression::StoreLocalVariable { name: n2, value: Box::new(rhs) },
-        Expression::Condition {
-            condition: Box::new(Expression::BinaryExpression {
-                lhs: a1.clone(),
-                rhs: a2.clone(),
-                op,
-            }),
-            true_expr: a1,
-            false_expr: a2,
-        },
-    ])
-}
