@@ -25,6 +25,10 @@ pub enum Type {
     Invalid,
     /// The type of an expression that return nothing
     Void,
+    /// The type of a property two way binding whose type was not yet inferred
+    InferredProperty,
+    /// The type of a callback alias whose type was not yet inferred
+    InferredCallback,
     Component(Rc<Component>),
     Builtin(Rc<BuiltinElement>),
     Native(Rc<NativeClass>),
@@ -54,7 +58,7 @@ pub enum Type {
     PathElements,
     Easing,
     Brush,
-
+    /// This is usually a model
     Array(Box<Type>),
     Struct {
         fields: BTreeMap<String, Type>,
@@ -69,6 +73,9 @@ pub enum Type {
     UnitProduct(Vec<(Unit, i8)>),
 
     ElementReference,
+
+    /// This is a `SharedArray<f32>`
+    LayoutCache,
 }
 
 impl core::cmp::PartialEq for Type {
@@ -76,6 +83,8 @@ impl core::cmp::PartialEq for Type {
         match self {
             Type::Invalid => matches!(other, Type::Invalid),
             Type::Void => matches!(other, Type::Void),
+            Type::InferredProperty => matches!(other, Type::InferredProperty),
+            Type::InferredCallback => matches!(other, Type::InferredCallback),
             Type::Component(a) => matches!(other, Type::Component(b) if Rc::ptr_eq(a, b)),
             Type::Builtin(a) => matches!(other, Type::Builtin(b) if Rc::ptr_eq(a, b)),
             Type::Native(a) => matches!(other, Type::Native(b) if Rc::ptr_eq(a, b)),
@@ -107,6 +116,7 @@ impl core::cmp::PartialEq for Type {
             Type::Enumeration(lhs) => matches!(other, Type::Enumeration(rhs) if lhs == rhs),
             Type::UnitProduct(a) => matches!(other, Type::UnitProduct(b) if a == b),
             Type::ElementReference => matches!(other, Type::ElementReference),
+            Type::LayoutCache => matches!(other, Type::LayoutCache),
         }
     }
 }
@@ -116,6 +126,8 @@ impl Display for Type {
         match self {
             Type::Invalid => write!(f, "<error>"),
             Type::Void => write!(f, "void"),
+            Type::InferredProperty => write!(f, "?"),
+            Type::InferredCallback => write!(f, "callback"),
             Type::Component(c) => c.id.fmt(f),
             Type::Builtin(b) => b.name.fmt(f),
             Type::Native(b) => b.class_name.fmt(f),
@@ -189,15 +201,12 @@ impl Display for Type {
                 write!(f, "({})", x.join("×"))
             }
             Type::ElementReference => write!(f, "element ref"),
+            Type::LayoutCache => write!(f, "layout cache"),
         }
     }
 }
 
 impl Type {
-    pub fn is_object_type(&self) -> bool {
-        matches!(self, Self::Component(_) | Self::Builtin(_))
-    }
-
     /// valid type for properties
     pub fn is_property_type(&self) -> bool {
         matches!(
@@ -220,6 +229,7 @@ impl Type {
                 | Self::Struct { .. }
                 | Self::Array(_)
                 | Self::Brush
+                | Self::InferredProperty
         )
     }
 
@@ -238,15 +248,16 @@ impl Type {
                     } else {
                         Cow::Borrowed(name)
                     };
-                let property_type =
-                    b.properties.get(resolved_name.as_ref()).cloned().unwrap_or_else(|| {
+                match b.properties.get(resolved_name.as_ref()) {
+                    None => {
                         if b.is_non_item_type {
-                            Type::Invalid
+                            PropertyLookupResult { resolved_name, property_type: Type::Invalid }
                         } else {
-                            crate::typeregister::reserved_property(resolved_name.as_ref())
+                            crate::typeregister::reserved_property(name)
                         }
-                    });
-                PropertyLookupResult { resolved_name, property_type }
+                    }
+                    Some(p) => PropertyLookupResult { resolved_name, property_type: p.ty.clone() },
+                }
             }
             Type::Native(n) => {
                 let resolved_name = if let Some(alias_name) = n.lookup_alias(name.as_ref()) {
@@ -278,7 +289,9 @@ impl Type {
                 );
                 r
             }
-            Type::Builtin(b) => b.properties.iter().map(|(k, t)| (k.clone(), t.clone())).collect(),
+            Type::Builtin(b) => {
+                b.properties.iter().map(|(k, t)| (k.clone(), t.ty.clone())).collect()
+            }
             Type::Native(n) => n.properties.iter().map(|(k, t)| (k.clone(), t.clone())).collect(),
             _ => Vec::new(),
         }
@@ -353,7 +366,7 @@ impl Type {
         }
     }
 
-    /// Assime it is a Component, panic if it isn't
+    /// Assume it is a Component, panic if it isn't
     pub fn as_component(&self) -> &Rc<Component> {
         match self {
             Type::Component(c) => c,
@@ -453,6 +466,7 @@ impl Type {
             Type::Angle => Some(Unit::Deg),
             Type::Invalid => None,
             Type::Void => None,
+            Type::InferredProperty | Type::InferredCallback => None,
             Type::Component(_) => None,
             Type::Builtin(_) => None,
             Type::Native(_) => None,
@@ -473,6 +487,7 @@ impl Type {
             Type::Enumeration(_) => None,
             Type::UnitProduct(_) => None,
             Type::ElementReference => None,
+            Type::LayoutCache => None,
         }
     }
 
@@ -492,11 +507,31 @@ impl Default for Type {
     }
 }
 
+/// Information about properties in NativeClass
+#[derive(Debug, Clone)]
+pub struct BuiltinPropertyInfo {
+    /// The property type
+    pub ty: Type,
+    /// When set, this is the initial value that we will have to set if no other binding were specified
+    pub default_value: Option<Expression>,
+    /// Most properties are just set from the .60 code and never modified by the native code.
+    /// But some properties, such as `TouchArea::pressed` are being set by the native code, these
+    /// are output properties which are meant to be read by the .60.
+    /// `is_native_output` is true if the native item can modify the property.
+    pub is_native_output: bool,
+}
+
+impl BuiltinPropertyInfo {
+    pub fn new(ty: Type) -> Self {
+        Self { ty, default_value: None, is_native_output: false }
+    }
+}
+
 #[derive(Debug, Clone, Default)]
 pub struct NativeClass {
     pub parent: Option<Rc<NativeClass>>,
     pub class_name: String,
-    pub vtable_symbol: String,
+    pub cpp_vtable_getter: String,
     pub properties: HashMap<String, Type>,
     pub deprecated_aliases: HashMap<String, String>,
     pub cpp_type: Option<String>,
@@ -505,10 +540,10 @@ pub struct NativeClass {
 
 impl NativeClass {
     pub fn new(class_name: &str) -> Self {
-        let vtable_symbol = format!("{}VTable", class_name);
+        let cpp_vtable_getter = format!("SIXTYFPS_GET_ITEM_VTABLE({}VTable)", class_name);
         Self {
             class_name: class_name.into(),
-            vtable_symbol,
+            cpp_vtable_getter,
             properties: Default::default(),
             ..Default::default()
         }
@@ -527,10 +562,6 @@ impl NativeClass {
         self.properties.len() + self.parent.clone().map(|p| p.property_count()).unwrap_or_default()
     }
 
-    pub fn local_property_iter(&self) -> impl Iterator<Item = (&String, &Type)> {
-        self.properties.iter()
-    }
-
     pub fn visit_class_hierarchy(self: Rc<Self>, mut visitor: impl FnMut(&Rc<Self>)) {
         visitor(&self);
         if let Some(parent_class) = &self.parent {
@@ -559,41 +590,6 @@ impl NativeClass {
             None
         }
     }
-
-    fn lookup_property_distance(self: Rc<Self>, name: &str) -> (usize, Rc<Self>) {
-        let mut distance = 0;
-        let mut class = self;
-        loop {
-            if class.properties.contains_key(name) {
-                return (distance, class);
-            }
-            distance += 1;
-            class = class.parent.as_ref().unwrap().clone();
-        }
-    }
-
-    pub fn select_minimal_class_based_on_property_usage<'a>(
-        self: Rc<Self>,
-        properties_used: impl Iterator<Item = &'a String>,
-    ) -> Rc<Self> {
-        let mut minimal_class = self.clone();
-        while let Some(class) = minimal_class.parent.clone() {
-            minimal_class = class;
-        }
-        let (_min_distance, minimal_class) = properties_used.fold(
-            (std::usize::MAX, minimal_class),
-            |(current_distance, current_class), prop_name| {
-                let (prop_distance, prop_class) = self.clone().lookup_property_distance(&prop_name);
-
-                if prop_distance < current_distance {
-                    (prop_distance, prop_class)
-                } else {
-                    (current_distance, current_class)
-                }
-            },
-        );
-        minimal_class
-    }
 }
 
 #[derive(Debug, Clone)]
@@ -616,8 +612,7 @@ impl Default for DefaultSizeBinding {
 pub struct BuiltinElement {
     pub name: String,
     pub native_class: Rc<NativeClass>,
-    pub properties: HashMap<String, Type>,
-    pub default_bindings: HashMap<String, Expression>,
+    pub properties: HashMap<String, BuiltinPropertyInfo>,
     pub additional_accepted_child_types: HashMap<String, Type>,
     pub disallow_global_types_as_child_elements: bool,
     /// Non-item type do not have reserved properties (x/width/rowspan/...) added to them  (eg: PropertyAnimation)
@@ -634,8 +629,8 @@ impl BuiltinElement {
     pub fn new(native_class: Rc<NativeClass>) -> Self {
         let mut properties = HashMap::new();
         native_class.clone().visit_class_hierarchy(|class| {
-            for (prop_name, prop_type) in class.local_property_iter() {
-                properties.insert(prop_name.clone(), prop_type.clone());
+            for (prop_name, prop_type) in &class.properties {
+                properties.insert(prop_name.clone(), BuiltinPropertyInfo::new(prop_type.clone()));
             }
         });
         Self {
@@ -647,45 +642,18 @@ impl BuiltinElement {
     }
 }
 
-#[test]
-fn test_select_minimal_class_based_on_property_usage() {
-    let first = Rc::new(NativeClass::new_with_properties(
-        "first_class",
-        [("first_prop".to_owned(), Type::Int32)].iter().cloned(),
-    ));
-
-    let mut second = NativeClass::new_with_properties(
-        "second_class",
-        [("second_prop".to_owned(), Type::Int32)].iter().cloned(),
-    );
-    second.parent = Some(first.clone());
-    let second = Rc::new(second);
-
-    let reduce_to_first = second
-        .clone()
-        .select_minimal_class_based_on_property_usage(["first_prop".to_owned()].iter());
-
-    assert_eq!(reduce_to_first.class_name, first.class_name);
-
-    let reduce_to_second = second
-        .clone()
-        .select_minimal_class_based_on_property_usage(["second_prop".to_owned()].iter());
-
-    assert_eq!(reduce_to_second.class_name, second.class_name);
-
-    let reduce_to_second = second.clone().select_minimal_class_based_on_property_usage(
-        ["first_prop".to_owned(), "second_prop".to_owned()].iter(),
-    );
-
-    assert_eq!(reduce_to_second.class_name, second.class_name);
-}
-
 #[derive(PartialEq, Debug)]
 pub struct PropertyLookupResult<'a> {
     pub resolved_name: std::borrow::Cow<'a, str>,
     pub property_type: Type,
 }
 
+impl<'a> PropertyLookupResult<'a> {
+    pub fn is_valid(&self) -> bool {
+        self.property_type != Type::Invalid
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct Enumeration {
     pub name: String,
