@@ -12,8 +12,10 @@ LICENSE END */
 
 use std::rc::Rc;
 
-use crate::diagnostics::BuildDiagnostics;
-use crate::expression_tree::{BuiltinMacroFunction, EasingCurve, Expression, Unit};
+use crate::diagnostics::{BuildDiagnostics, Spanned};
+use crate::expression_tree::{
+    BuiltinFunction, BuiltinMacroFunction, EasingCurve, Expression, Unit,
+};
 use crate::langtype::{Enumeration, EnumerationValue, Type};
 use crate::namedreference::NamedReference;
 use crate::object_tree::{find_parent_element, ElementRc};
@@ -89,8 +91,8 @@ impl LookupResult {
 
 /// Represent an object which has properties which can be accessible
 pub trait LookupObject {
-    /// Will call the function for each entry (usefull for completion)
-    /// If the function return Some, it will imediatly be returned and not called further
+    /// Will call the function for each entry (useful for completion)
+    /// If the function return Some, it will immediately be returned and not called further
     fn for_each_entry<R>(
         &self,
         ctx: &LookupCtx,
@@ -98,7 +100,7 @@ pub trait LookupObject {
     ) -> Option<R>;
 
     /// Perform a lookup of a given identifier.
-    /// One does not have to re-implment unless we can make it faster
+    /// One does not have to re-implement unless we can make it faster
     fn lookup(&self, ctx: &LookupCtx, name: &str) -> Option<LookupResult> {
         self.for_each_entry(ctx, &mut |prop, expr| (prop == name).then(|| LookupResult::new(expr)))
     }
@@ -276,6 +278,12 @@ impl LookupObject for ElementRc {
                 return Some(r);
             }
         }
+        for (name, ty) in crate::typeregister::reserved_properties() {
+            let e = expression_from_reference(NamedReference::new(self, &name), &ty);
+            if let Some(r) = f(&name, e) {
+                return Some(r);
+            }
+        }
         None
     }
 
@@ -444,25 +452,26 @@ impl LookupObject for BuiltinFunctionLookup {
         ctx: &LookupCtx,
         f: &mut impl FnMut(&str, Expression) -> Option<R>,
     ) -> Option<R> {
-        use crate::expression_tree::BuiltinFunction;
         use Expression::{BuiltinFunctionReference, BuiltinMacroReference};
         let t = &ctx.current_token;
-        None.or_else(|| f("debug", BuiltinFunctionReference(BuiltinFunction::Debug)))
-            .or_else(|| f("mod", BuiltinFunctionReference(BuiltinFunction::Mod)))
-            .or_else(|| f("round", BuiltinFunctionReference(BuiltinFunction::Round)))
-            .or_else(|| f("ceil", BuiltinFunctionReference(BuiltinFunction::Ceil)))
-            .or_else(|| f("floor", BuiltinFunctionReference(BuiltinFunction::Floor)))
-            .or_else(|| f("sqrt", BuiltinFunctionReference(BuiltinFunction::Sqrt)))
+        let sl = || t.as_ref().map(|t| t.to_source_location());
+        None.or_else(|| f("debug", BuiltinMacroReference(BuiltinMacroFunction::Debug, t.clone())))
+            .or_else(|| f("mod", BuiltinFunctionReference(BuiltinFunction::Mod, sl())))
+            .or_else(|| f("round", BuiltinFunctionReference(BuiltinFunction::Round, sl())))
+            .or_else(|| f("ceil", BuiltinFunctionReference(BuiltinFunction::Ceil, sl())))
+            .or_else(|| f("floor", BuiltinFunctionReference(BuiltinFunction::Floor, sl())))
+            .or_else(|| f("abs", BuiltinFunctionReference(BuiltinFunction::Abs, sl())))
+            .or_else(|| f("sqrt", BuiltinFunctionReference(BuiltinFunction::Sqrt, sl())))
             .or_else(|| f("rgb", BuiltinMacroReference(BuiltinMacroFunction::Rgb, t.clone())))
             .or_else(|| f("rgba", BuiltinMacroReference(BuiltinMacroFunction::Rgb, t.clone())))
             .or_else(|| f("max", BuiltinMacroReference(BuiltinMacroFunction::Max, t.clone())))
             .or_else(|| f("min", BuiltinMacroReference(BuiltinMacroFunction::Min, t.clone())))
-            .or_else(|| f("sin", BuiltinFunctionReference(BuiltinFunction::Sin)))
-            .or_else(|| f("cos", BuiltinFunctionReference(BuiltinFunction::Cos)))
-            .or_else(|| f("tan", BuiltinFunctionReference(BuiltinFunction::Tan)))
-            .or_else(|| f("asin", BuiltinFunctionReference(BuiltinFunction::ASin)))
-            .or_else(|| f("acos", BuiltinFunctionReference(BuiltinFunction::ACos)))
-            .or_else(|| f("atan", BuiltinFunctionReference(BuiltinFunction::ATan)))
+            .or_else(|| f("sin", BuiltinFunctionReference(BuiltinFunction::Sin, sl())))
+            .or_else(|| f("cos", BuiltinFunctionReference(BuiltinFunction::Cos, sl())))
+            .or_else(|| f("tan", BuiltinFunctionReference(BuiltinFunction::Tan, sl())))
+            .or_else(|| f("asin", BuiltinFunctionReference(BuiltinFunction::ASin, sl())))
+            .or_else(|| f("acos", BuiltinFunctionReference(BuiltinFunction::ACos, sl())))
+            .or_else(|| f("atan", BuiltinFunctionReference(BuiltinFunction::ATan, sl())))
     }
 }
 
@@ -494,8 +503,8 @@ impl LookupObject for Expression {
                     None
                 }
             }
-            _ => {
-                if let Type::Struct { fields, .. } = self.ty() {
+            _ => match self.ty() {
+                Type::Struct { fields, .. } => {
                     for name in fields.keys() {
                         if let Some(r) = f(
                             &name,
@@ -507,9 +516,14 @@ impl LookupObject for Expression {
                             return Some(r);
                         }
                     }
+                    None
                 }
-                None
-            }
+                Type::Component(c) => c.root_element.for_each_entry(ctx, f),
+                Type::String => StringExpression(self).for_each_entry(ctx, f),
+                Type::Color => ColorExpression(self).for_each_entry(ctx, f),
+                Type::Image => ImageExpression(self).for_each_entry(ctx, f),
+                _ => None,
+            },
         }
     }
 
@@ -523,17 +537,81 @@ impl LookupObject for Expression {
                     None
                 }
             }
-            _ => {
-                if let Type::Struct { fields, .. } = self.ty() {
-                    if fields.contains_key(name) {
-                        return Some(LookupResult::new(Expression::StructFieldAccess {
-                            base: Box::new(self.clone()),
-                            name: name.to_string(),
-                        }));
-                    }
-                }
-                None
-            }
+            _ => match self.ty() {
+                Type::Struct { fields, .. } => fields.contains_key(name).then(|| {
+                    LookupResult::new(Expression::StructFieldAccess {
+                        base: Box::new(self.clone()),
+                        name: name.to_string(),
+                    })
+                }),
+                Type::Component(c) => c.root_element.lookup(ctx, name),
+                Type::String => StringExpression(self).lookup(ctx, name),
+                Type::Color => ColorExpression(self).lookup(ctx, name),
+                Type::Image => ImageExpression(self).lookup(ctx, name),
+                _ => None,
+            },
         }
     }
 }
+
+struct StringExpression<'a>(&'a Expression);
+impl<'a> LookupObject for StringExpression<'a> {
+    fn for_each_entry<R>(
+        &self,
+        ctx: &LookupCtx,
+        f: &mut impl FnMut(&str, Expression) -> Option<R>,
+    ) -> Option<R> {
+        let member_function = |f: BuiltinFunction| Expression::MemberFunction {
+            base: Box::new(self.0.clone()),
+            base_node: ctx.current_token.clone(), // Note that this is not the base_node, but the function's node
+            member: Box::new(Expression::BuiltinFunctionReference(
+                f,
+                ctx.current_token.as_ref().map(|t| t.to_source_location()),
+            )),
+        };
+        None.or_else(|| f("is_float", member_function(BuiltinFunction::StringIsFloat)))
+            .or_else(|| f("to_float", member_function(BuiltinFunction::StringToFloat)))
+    }
+}
+struct ColorExpression<'a>(&'a Expression);
+impl<'a> LookupObject for ColorExpression<'a> {
+    fn for_each_entry<R>(
+        &self,
+        ctx: &LookupCtx,
+        f: &mut impl FnMut(&str, Expression) -> Option<R>,
+    ) -> Option<R> {
+        let member_function = |f: BuiltinFunction| Expression::MemberFunction {
+            base: Box::new(self.0.clone()),
+            base_node: ctx.current_token.clone(), // Note that this is not the base_node, but the function's node
+            member: Box::new(Expression::BuiltinFunctionReference(
+                f,
+                ctx.current_token.as_ref().map(|t| t.to_source_location()),
+            )),
+        };
+        None.or_else(|| f("brighter", member_function(BuiltinFunction::ColorBrighter)))
+            .or_else(|| f("darker", member_function(BuiltinFunction::ColorDarker)))
+    }
+}
+
+struct ImageExpression<'a>(&'a Expression);
+impl<'a> LookupObject for ImageExpression<'a> {
+    fn for_each_entry<R>(
+        &self,
+        ctx: &LookupCtx,
+        f: &mut impl FnMut(&str, Expression) -> Option<R>,
+    ) -> Option<R> {
+        let field_access = |f: &str| Expression::StructFieldAccess {
+            base: Box::new(Expression::FunctionCall {
+                function: Box::new(Expression::BuiltinFunctionReference(
+                    BuiltinFunction::ImageSize,
+                    ctx.current_token.as_ref().map(|t| t.to_source_location()),
+                )),
+                source_location: ctx.current_token.as_ref().map(|t| t.to_source_location()),
+                arguments: vec![self.0.clone()],
+            }),
+            name: f.into(),
+        };
+        None.or_else(|| f("width", field_access("width")))
+            .or_else(|| f("height", field_access("height")))
+    }
+}
