@@ -13,16 +13,17 @@ LICENSE END */
 
 use itertools::Either;
 
-use crate::diagnostics::{BuildDiagnostics, Spanned};
+use crate::diagnostics::{BuildDiagnostics, SourceLocation, Spanned};
 use crate::expression_tree::{self, BindingExpression, Expression, Unit};
 use crate::langtype::PropertyLookupResult;
 use crate::langtype::{BuiltinElement, NativeClass, Type};
+use crate::layout::{LayoutConstraints, Orientation};
 use crate::namedreference::NamedReference;
 use crate::parser::{identifier_text, syntax_nodes, SyntaxKind, SyntaxNode};
 use crate::typeloader::ImportedTypes;
 use crate::typeregister::TypeRegister;
 use std::cell::RefCell;
-use std::collections::HashMap;
+use std::collections::{BTreeMap, HashMap};
 use std::rc::{Rc, Weak};
 
 macro_rules! unwrap_or_continue {
@@ -30,7 +31,7 @@ macro_rules! unwrap_or_continue {
         match $e {
             Some(x) => x,
             None => {
-                debug_assert!($diag.has_error()); // error should have been reported at parseing time
+                debug_assert!($diag.has_error()); // error should have been reported at parsing time
                 continue;
             }
         }
@@ -131,7 +132,10 @@ impl Document {
         let custom_fonts = foreign_imports
             .into_iter()
             .filter_map(|import| {
-                if import.file.ends_with(".ttc") || import.file.ends_with(".ttf") {
+                if import.file.ends_with(".ttc")
+                    || import.file.ends_with(".ttf")
+                    || import.file.ends_with(".otf")
+                {
                     Some(import.file)
                 } else {
                     diag.push_error(
@@ -188,8 +192,8 @@ pub struct Component {
     /// generator for symbol generation.
     pub embedded_file_resources: RefCell<HashMap<String, usize>>,
 
-    /// All layouts in this component
-    pub layouts: RefCell<crate::layout::LayoutVec>,
+    /// The layout constraints of the root item
+    pub root_constraints: RefCell<LayoutConstraints>,
 
     /// When creating this component and inserting "children", append them to the children of
     /// the element pointer to by this field.
@@ -258,7 +262,10 @@ impl PropertyDeclaration {
     // For diagnostics: return a node pointing to the type
     pub fn type_node(&self) -> Option<SyntaxNode> {
         self.node.as_ref().map(|x| -> crate::parser::SyntaxNode {
-            x.as_ref().either(|x| x.Type().into(), |x| x.clone().into())
+            x.as_ref().either(
+                |x| x.Type().map_or_else(|| x.clone().into(), |x| x.into()),
+                |x| x.clone().into(),
+            )
         })
     }
 }
@@ -275,13 +282,13 @@ pub struct TransitionPropertyAnimation {
     pub state_id: i32,
     /// false for 'to', true for 'out'
     pub is_out: bool,
-    /// The content of the `anumation` object
+    /// The content of the `animation` object
     pub animation: ElementRc,
 }
 
 impl TransitionPropertyAnimation {
     /// Return an expression which returns a boolean which is true if the transition is active.
-    /// The state argument is an expresison referencing the state property of type StateInfo
+    /// The state argument is an expression referencing the state property of type StateInfo
     pub fn condition(&self, state: Expression) -> Expression {
         Expression::BinaryExpression {
             lhs: Box::new(Expression::StructFieldAccess {
@@ -300,7 +307,7 @@ pub enum PropertyAnimation {
     Transition { state_ref: Expression, animations: Vec<TransitionPropertyAnimation> },
 }
 
-/// An Element is an instentation of a Component
+/// An Element is an instantiation of a Component
 #[derive(Default)]
 pub struct Element {
     /// The id as named in the original .60 file.
@@ -312,12 +319,14 @@ pub struct Element {
     //pub base: QualifiedTypeName,
     pub base_type: crate::langtype::Type,
     /// Currently contains also the callbacks. FIXME: should that be changed?
-    pub bindings: HashMap<String, BindingExpression>,
+    pub bindings: BTreeMap<String, BindingExpression>,
+    pub property_analysis: RefCell<HashMap<String, PropertyAnalysis>>,
+
     pub children: Vec<ElementRc>,
     /// The component which contains this element.
     pub enclosing_component: Weak<Component>,
 
-    pub property_declarations: HashMap<String, PropertyDeclaration>,
+    pub property_declarations: BTreeMap<String, PropertyDeclaration>,
 
     /// Main owner for a reference to a property.
     pub named_references: crate::namedreference::NamedReferenceContainer,
@@ -332,6 +341,8 @@ pub struct Element {
 
     /// true when this item's geometry is handled by a layout
     pub child_of_layout: bool,
+    /// The property pointing to the layout info. `(horizontal, vertical)`
+    pub layout_info_prop: Option<(NamedReference, NamedReference)>,
 
     /// true if this Element is the fake Flickable viewport
     pub is_flickable_viewport: bool,
@@ -356,41 +367,83 @@ impl Spanned for Element {
 
 impl core::fmt::Debug for Element {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        if let Some(repeated) = &self.repeated {
-            write!(f, "for {}[{}] in ", repeated.model_data_id, repeated.index_id)?;
-            expression_tree::pretty_print(f, &repeated.model)?;
-            write!(f, ":")?;
-        }
-        writeln!(f, "{} := {} {{", self.id, self.base_type)?;
-        for (name, ty) in &self.property_declarations {
-            if let Some(alias) = &ty.is_alias {
-                writeln!(f, "  alias<{}> {} <=> {:?};", ty.property_type, name, alias)?
-            } else {
-                writeln!(f, "  property<{}> {};", ty.property_type, name)?
-            }
-        }
-        for (name, expr) in &self.bindings {
-            write!(f, "   {}: ", name)?;
-            expression_tree::pretty_print(f, &expr.expression)?;
-            writeln!(f, "; /*{}*/", expr.priority)?;
-        }
-        for (name, anim) in &self.property_animations {
-            writeln!(f, "  animate {} {:?}", name, anim)?;
-        }
-        if !self.states.is_empty() {
-            writeln!(f, "  states {:?}", self.states)?;
-        }
-        if !self.transitions.is_empty() {
-            writeln!(f, "  transitions {:?} ", self.transitions)?;
-        }
-        for c in &self.children {
-            writeln!(f, "  {:?}", *c.borrow())?;
-        }
+        pretty_print(f, self, 0)
+    }
+}
 
-        if let Type::Component(base) = &self.base_type {
-            writeln!(f, "{:?}", base.root_element)?;
+pub fn pretty_print(
+    f: &mut impl std::fmt::Write,
+    e: &Element,
+    indentation: usize,
+) -> std::fmt::Result {
+    if let Some(repeated) = &e.repeated {
+        write!(f, "for {}[{}] in ", repeated.model_data_id, repeated.index_id)?;
+        expression_tree::pretty_print(f, &repeated.model)?;
+        write!(f, ":")?;
+    }
+    writeln!(f, "{} := {} {{", e.id, e.base_type)?;
+    let mut indentation = indentation + 1;
+    macro_rules! indent {
+        () => {
+            for _ in 0..indentation {
+                write!(f, "   ")?
+            }
+        };
+    }
+    for (name, ty) in &e.property_declarations {
+        indent!();
+        if let Some(alias) = &ty.is_alias {
+            writeln!(f, "alias<{}> {} <=> {:?};", ty.property_type, name, alias)?
+        } else {
+            writeln!(f, "property<{}> {};", ty.property_type, name)?
         }
-        writeln!(f, "}}")
+    }
+    for (name, expr) in &e.bindings {
+        indent!();
+        write!(f, "{}: ", name)?;
+        expression_tree::pretty_print(f, &expr.expression)?;
+        writeln!(f, ";")?;
+        //writeln!(f, "; /*{}*/", expr.priority)?;
+    }
+    for (name, anim) in &e.property_animations {
+        indent!();
+        writeln!(f, "animate {} {:?}", name, anim)?;
+    }
+    if !e.states.is_empty() {
+        indent!();
+        writeln!(f, "states {:?}", e.states)?;
+    }
+    if !e.transitions.is_empty() {
+        indent!();
+        writeln!(f, "transitions {:?} ", e.transitions)?;
+    }
+    for c in &e.children {
+        indent!();
+        pretty_print(f, &c.borrow(), indentation)?
+    }
+
+    /*if let Type::Component(base) = &e.base_type {
+        pretty_print(f, &c.borrow(), indentation)?
+    }*/
+    indentation -= 1;
+    indent!();
+    writeln!(f, "}}")
+}
+
+#[derive(Clone, Default, Debug)]
+pub struct PropertyAnalysis {
+    /// true if somewhere in the code, there is an expression that changes this property with an assignment
+    pub is_set: bool,
+
+    /// true if somewhere in the code, an expression is reading this property
+    /// Note: currently this is only set in the binding analysis pass
+    pub is_read: bool,
+}
+
+impl PropertyAnalysis {
+    /// Merge analysis from base element for e.g. inlining
+    pub fn merge(&mut self, other: &PropertyAnalysis) {
+        self.is_set |= other.is_set;
     }
 }
 
@@ -409,9 +462,9 @@ pub struct RepeatedElementInfo {
     pub model: Expression,
     pub model_data_id: String,
     pub index_id: String,
-    /// A conditional element is just a for whose model is a bolean expression
+    /// A conditional element is just a for whose model is a boolean expression
     ///
-    /// When this is true, the model is of type bolean instead of Model
+    /// When this is true, the model is of type boolean instead of Model
     pub is_conditional_element: bool,
     /// When the for is the delegate of a ListView
     pub is_listview: Option<ListViewInfo>,
@@ -428,26 +481,27 @@ impl Element {
         diag: &mut BuildDiagnostics,
         tr: &TypeRegister,
     ) -> ElementRc {
-        let (base_type, name_for_looup_errors) = if let Some(base_node) = node.QualifiedName() {
+        let base_type = if let Some(base_node) = node.QualifiedName() {
             let base = QualifiedTypeName::from_node(base_node.clone());
             let base_string = base.to_string();
-            let base_type = match parent_type.lookup_type_for_child_element(&base_string, tr) {
-                Ok(ty) => ty,
-                Err(err) => {
-                    diag.push_error(err, &base_node);
-                    return ElementRc::default();
-                }
-            };
-            assert!(base_type.is_object_type());
-            if let Type::Component(c) = &base_type {
-                if c.is_global() {
+            match parent_type.lookup_type_for_child_element(&base_string, tr) {
+                Ok(Type::Component(c)) if c.is_global() => {
                     diag.push_error(
                         "Cannot create an instance of a global component".into(),
                         &base_node,
-                    )
+                    );
+                    Type::Invalid
+                }
+                Ok(ty @ Type::Component(_)) | Ok(ty @ Type::Builtin(_)) => ty,
+                Ok(ty) => {
+                    diag.push_error(format!("'{}' cannot be used as an element", ty), &base_node);
+                    Type::Invalid
+                }
+                Err(err) => {
+                    diag.push_error(err, &base_node);
+                    Type::Invalid
                 }
             }
-            (base_type, format!(" in {}", base))
         } else {
             if parent_type != Type::Invalid {
                 // This should normally never happen because the parser does not allow for this
@@ -466,20 +520,33 @@ impl Element {
             node.PropertyAnimation().for_each(|n| error_on(&n, "animations"));
             node.States().for_each(|n| error_on(&n, "states"));
             node.Transitions().for_each(|n| error_on(&n, "transitions"));
-            (Type::Void, String::new())
+            Type::Void
         };
         let mut r = Element { id, base_type, node: Some(node.clone()), ..Default::default() };
 
         for prop_decl in node.PropertyDeclaration() {
-            let type_node = prop_decl.Type();
-            let prop_type = type_from_node(type_node.clone(), diag, tr);
+            let prop_type = prop_decl
+                .Type()
+                .map(|type_node| {
+                    let prop_type = type_from_node(type_node.clone(), diag, tr);
+
+                    if prop_type != Type::Invalid && !prop_type.is_property_type() {
+                        diag.push_error(
+                            format!("'{}' is not a valid property type", prop_type),
+                            &type_node,
+                        );
+                    }
+                    prop_type
+                })
+                // Type::Void is used for two way bindings without type specified
+                .unwrap_or(Type::InferredProperty);
+
             let unresolved_prop_name =
                 unwrap_or_continue!(identifier_text(&prop_decl.DeclaredIdentifier()); diag);
             let PropertyLookupResult {
                 resolved_name: prop_name,
                 property_type: maybe_existing_prop_type,
             } = r.lookup_property(&unresolved_prop_name);
-
             if !matches!(maybe_existing_prop_type, Type::Invalid) {
                 diag.push_error(
                     format!("Cannot override property '{}'", prop_name),
@@ -487,13 +554,6 @@ impl Element {
                 )
             }
 
-            if prop_type != Type::Invalid && !prop_type.is_property_type() {
-                diag.push_error(
-                    format!("'{}' is not a valid property type", prop_type),
-                    &type_node,
-                );
-            }
-
             r.property_declarations.insert(
                 prop_name.to_string(),
                 PropertyDeclaration {
@@ -528,27 +588,41 @@ impl Element {
         }
 
         r.parse_bindings(
-            &name_for_looup_errors,
             node.Binding().filter_map(|b| {
                 Some((b.child_token(SyntaxKind::Identifier)?, b.BindingExpression().into()))
             }),
             diag,
         );
         r.parse_bindings(
-            &name_for_looup_errors,
             node.TwoWayBinding()
                 .filter_map(|b| Some((b.child_token(SyntaxKind::Identifier)?, b.into()))),
             diag,
         );
 
         if let Type::Builtin(builtin_base) = &r.base_type {
-            for (prop, expr) in &builtin_base.default_bindings {
-                r.bindings.entry(prop.clone()).or_insert_with(|| expr.clone().into());
+            for (prop, info) in &builtin_base.properties {
+                if let Some(expr) = &info.default_value {
+                    r.bindings.entry(prop.clone()).or_insert_with(|| expr.clone().into());
+                }
             }
         }
 
         for sig_decl in node.CallbackDeclaration() {
             let name = unwrap_or_continue!(identifier_text(&sig_decl.DeclaredIdentifier()); diag);
+
+            if let Some(csn) = sig_decl.TwoWayBinding() {
+                r.bindings.insert(name.clone(), BindingExpression::new_uncompiled(csn.into()));
+                r.property_declarations.insert(
+                    name,
+                    PropertyDeclaration {
+                        property_type: Type::InferredCallback,
+                        node: Some(Either::Right(sig_decl)),
+                        ..Default::default()
+                    },
+                );
+                continue;
+            }
+
             let args = sig_decl.Type().map(|node_ty| type_from_node(node_ty, diag, tr)).collect();
             let return_type = sig_decl
                 .ReturnType()
@@ -567,7 +641,7 @@ impl Element {
             let unresolved_name = unwrap_or_continue!(identifier_text(&con_node); diag);
             let PropertyLookupResult { resolved_name, property_type } =
                 r.lookup_property(&unresolved_name);
-            if let Type::Callback { args, .. } = property_type {
+            if let Type::Callback { args, .. } = &property_type {
                 let num_arg = con_node.DeclaredIdentifier().count();
                 if num_arg > args.len() {
                     diag.push_error(
@@ -580,21 +654,24 @@ impl Element {
                         &con_node.child_token(SyntaxKind::Identifier).unwrap(),
                     );
                 }
-                if r.bindings
-                    .insert(
-                        resolved_name.into_owned(),
-                        BindingExpression::new_uncompiled(con_node.clone().into()),
-                    )
-                    .is_some()
-                {
-                    diag.push_error(
-                        "Duplicated callback".into(),
-                        &con_node.child_token(SyntaxKind::Identifier).unwrap(),
-                    );
-                }
+            } else if property_type == Type::InferredCallback {
+                // argument matching will happen later
             } else {
                 diag.push_error(
-                    format!("'{}' is not a callback{}", unresolved_name, name_for_looup_errors),
+                    format!("'{}' is not a callback in {}", unresolved_name, r.base_type),
+                    &con_node.child_token(SyntaxKind::Identifier).unwrap(),
+                );
+                continue;
+            }
+            if r.bindings
+                .insert(
+                    resolved_name.into_owned(),
+                    BindingExpression::new_uncompiled(con_node.clone().into()),
+                )
+                .is_some()
+            {
+                diag.push_error(
+                    "Duplicated callback".into(),
                     &con_node.child_token(SyntaxKind::Identifier).unwrap(),
                 );
             }
@@ -651,26 +728,14 @@ impl Element {
 
         for se in node.children() {
             if se.kind() == SyntaxKind::SubElement {
-                let id = identifier_text(&se).unwrap_or_default();
-                if matches!(id.as_ref(), "parent" | "self" | "root") {
-                    diag.push_error(
-                        format!("'{}' is a reserved id", id),
-                        &se.child_token(SyntaxKind::Identifier).unwrap(),
-                    )
-                }
-                if let Some(element_node) = se.child_node(SyntaxKind::Element) {
-                    let parent_type = r.borrow().base_type.clone();
-                    r.borrow_mut().children.push(Element::from_node(
-                        element_node.into(),
-                        id,
-                        parent_type,
-                        component_child_insertion_point,
-                        diag,
-                        tr,
-                    ));
-                } else {
-                    assert!(diag.has_error());
-                }
+                let parent_type = r.borrow().base_type.clone();
+                r.borrow_mut().children.push(Element::from_sub_element_node(
+                    se.into(),
+                    parent_type,
+                    component_child_insertion_point,
+                    diag,
+                    tr,
+                ));
             } else if se.kind() == SyntaxKind::RepeatedElement {
                 let rep = Element::from_repeated_node(
                     se.into(),
@@ -742,7 +807,7 @@ impl Element {
                         lookup_property_from_qualified_name(qn.clone(), &r, diag).and_then(
                             |(ne, prop_type)| {
                                 animation_element_from_node(&pa, &qn, prop_type, diag, tr)
-                                    .map(|anim_element| (ne, anim_element))
+                                    .map(|anim_element| (ne, qn.to_source_location(), anim_element))
                             },
                         )
                     })
@@ -755,6 +820,30 @@ impl Element {
         r
     }
 
+    fn from_sub_element_node(
+        node: syntax_nodes::SubElement,
+        parent_type: Type,
+        component_child_insertion_point: &mut Option<ChildrenInsertionPoint>,
+        diag: &mut BuildDiagnostics,
+        tr: &TypeRegister,
+    ) -> ElementRc {
+        let id = identifier_text(&node).unwrap_or_default();
+        if matches!(id.as_ref(), "parent" | "self" | "root") {
+            diag.push_error(
+                format!("'{}' is a reserved id", id),
+                &node.child_token(SyntaxKind::Identifier).unwrap(),
+            )
+        }
+        Element::from_node(
+            node.Element(),
+            id,
+            parent_type,
+            component_child_insertion_point,
+            diag,
+            tr,
+        )
+    }
+
     fn from_repeated_node(
         node: syntax_nodes::RepeatedElement,
         parent: &ElementRc,
@@ -783,10 +872,9 @@ impl Element {
             is_conditional_element: false,
             is_listview,
         };
-        let e = Element::from_node(
-            node.Element(),
-            String::new(),
-            parent.borrow().base_type.to_owned(),
+        let e = Element::from_sub_element_node(
+            node.SubElement(),
+            parent.borrow().base_type.clone(),
             component_child_insertion_point,
             diag,
             tr,
@@ -809,9 +897,8 @@ impl Element {
             is_conditional_element: true,
             is_listview: None,
         };
-        let e = Element::from_node(
-            node.Element(),
-            String::new(),
+        let e = Element::from_sub_element_node(
+            node.SubElement(),
             parent_type,
             component_child_insertion_point,
             diag,
@@ -838,7 +925,6 @@ impl Element {
 
     fn parse_bindings(
         &mut self,
-        name_for_lookup_error: &str,
         bindings: impl Iterator<Item = (crate::parser::SyntaxToken, SyntaxNode)>,
         diag: &mut BuildDiagnostics,
     ) {
@@ -850,14 +936,21 @@ impl Element {
                 diag.push_error(
                     match property_type {
                         Type::Invalid => {
-                            format!("Unknown property {}{}", unresolved_name, name_for_lookup_error)
+                            if self.base_type != Type::Invalid {
+                                format!(
+                                    "Unknown property {} in {}",
+                                    unresolved_name, self.base_type
+                                )
+                            } else {
+                                continue;
+                            }
                         }
                         Type::Callback { .. } => {
                             format!("'{}' is a callback. Use `=>` to connect", unresolved_name)
                         }
                         _ => format!(
-                            "Cannot assign to {}{} because it does not have a valid property type.",
-                            unresolved_name, name_for_lookup_error
+                            "Cannot assign to {} in {} because it does not have a valid property type",
+                            unresolved_name, self.base_type,
                         ),
                     },
                     &name_token,
@@ -908,6 +1001,13 @@ impl Element {
             }
         }
     }
+
+    pub fn layout_info_prop(&self, orientation: Orientation) -> Option<&NamedReference> {
+        self.layout_info_prop.as_ref().map(|prop| match orientation {
+            Orientation::Horizontal => &prop.0,
+            Orientation::Vertical => &prop.1,
+        })
+    }
 }
 
 /// Create a Type for this node
@@ -963,17 +1063,17 @@ fn animation_element_from_node(
     let anim_type = tr.property_animation_type_for_property(prop_type);
     if !matches!(anim_type, Type::Builtin(..)) {
         diag.push_error(
-            format!("'{}' is not an animatable property", prop_name.text().to_string().trim()),
+            format!(
+                "'{}' is not a property that can be animated",
+                prop_name.text().to_string().trim()
+            ),
             prop_name,
         );
         None
     } else {
-        let name_for_lookup_errors =
-            format!(" in {}", anim_type.as_builtin().native_class.class_name);
         let mut anim_element =
             Element { id: "".into(), base_type: anim_type, node: None, ..Default::default() };
         anim_element.parse_bindings(
-            &name_for_lookup_errors,
             anim.Binding().filter_map(|b| {
                 Some((b.child_token(SyntaxKind::Identifier)?, b.BindingExpression().into()))
             }),
@@ -1143,10 +1243,17 @@ pub fn recurse_elem_including_sub_components_no_borrow<State>(
     vis: &mut impl FnMut(&ElementRc, &State) -> State,
 ) {
     recurse_elem_no_borrow(&component.root_element, state, &mut |elem, state| {
-        if elem.borrow().repeated.is_some() {
+        let base = if elem.borrow().repeated.is_some() {
             if let Type::Component(base) = &elem.borrow().base_type {
-                recurse_elem_including_sub_components_no_borrow(base, state, vis);
+                Some(base.clone())
+            } else {
+                None
             }
+        } else {
+            None
+        };
+        if let Some(base) = base {
+            recurse_elem_including_sub_components_no_borrow(&base, state, vis);
         }
         vis(elem, state)
     });
@@ -1154,7 +1261,7 @@ pub fn recurse_elem_including_sub_components_no_borrow<State>(
         .popup_windows
         .borrow()
         .iter()
-        .for_each(|p| recurse_elem_including_sub_components(&p.component, state, vis))
+        .for_each(|p| recurse_elem_including_sub_components_no_borrow(&p.component, state, vis))
 }
 
 /// This visit the binding attached to this element, but does not recurse in children elements
@@ -1199,7 +1306,7 @@ pub fn visit_element_expressions(
 
     let mut transitions = std::mem::take(&mut elem.borrow_mut().transitions);
     for t in &mut transitions {
-        for (_, a) in &mut t.property_animations {
+        for (_, _, a) in &mut t.property_animations {
             visit_element_expressions_simple(a, &mut vis);
         }
     }
@@ -1231,6 +1338,9 @@ pub fn visit_all_named_references_in_element(
         match expr {
             Expression::PropertyReference(r) | Expression::CallbackReference(r) => vis(r),
             Expression::TwoWayBinding(r, _) => vis(r),
+            Expression::LayoutCacheAccess { layout_cache_prop, .. } => vis(layout_cache_prop),
+            Expression::SolveLayout(l, _) => l.visit_named_references(vis),
+            Expression::ComputeLayoutInfo(l, _) => l.visit_named_references(vis),
             // This is not really a named reference, but the result is the same, it need to be updated
             // FIXME: this should probably be lowered into a PropertyReference
             Expression::RepeaterModelReference { element }
@@ -1254,7 +1364,7 @@ pub fn visit_all_named_references_in_element(
     elem.borrow_mut().states = states;
     let mut transitions = std::mem::take(&mut elem.borrow_mut().transitions);
     for t in &mut transitions {
-        for (r, _) in &mut t.property_animations {
+        for (r, _, _) in &mut t.property_animations {
             vis(r)
         }
     }
@@ -1270,6 +1380,15 @@ pub fn visit_all_named_references_in_element(
         }
     }
     elem.borrow_mut().repeated = repeated;
+    let mut layout_info_prop = std::mem::take(&mut elem.borrow_mut().layout_info_prop);
+    layout_info_prop.as_mut().map(|(h, b)| (vis(h), vis(b)));
+    elem.borrow_mut().layout_info_prop = layout_info_prop;
+
+    let mut property_declarations = std::mem::take(&mut elem.borrow_mut().property_declarations);
+    for (_, pd) in &mut property_declarations {
+        pd.is_alias.as_mut().map(&mut vis);
+    }
+    elem.borrow_mut().property_declarations = property_declarations;
 }
 
 /// Visit all named reference in this component and sub component
@@ -1285,7 +1404,7 @@ pub fn visit_all_named_references(
             let compo = elem.borrow().enclosing_component.clone();
             if !Weak::ptr_eq(parent_compo, &compo) {
                 let compo = compo.upgrade().unwrap();
-                compo.layouts.borrow_mut().visit_named_references(vis);
+                compo.root_constraints.borrow_mut().visit_named_references(vis);
                 compo.popup_windows.borrow_mut().iter_mut().for_each(|p| {
                     vis(&mut p.x);
                     vis(&mut p.y);
@@ -1320,7 +1439,7 @@ pub struct Transition {
     /// false for 'to', true for 'out'
     pub is_out: bool,
     pub state_id: String,
-    pub property_animations: Vec<(NamedReference, ElementRc)>,
+    pub property_animations: Vec<(NamedReference, SourceLocation, ElementRc)>,
     /// Node pointing to the state name
     pub node: SyntaxNode,
 }
@@ -1438,7 +1557,7 @@ impl Exports {
 
 /// This function replace the root element of a repeated element. the previous root becomes the only
 /// child of the new root element.
-/// Note that no reference to the base component must exist ourside of repeated_element.base_type
+/// Note that no reference to the base component must exist outside of repeated_element.base_type
 pub fn inject_element_as_repeated_element(repeated_element: &ElementRc, new_root: ElementRc) {
     let component = repeated_element.borrow().base_type.as_component().clone();
     // Since we're going to replace the repeated element's component, we need to assert that
