@@ -23,6 +23,7 @@ pub use smol_str::SmolStr;
 use std::convert::TryFrom;
 
 mod document;
+mod element;
 mod expressions;
 mod statements;
 mod r#type;
@@ -322,15 +323,15 @@ declare_syntax! {
         Element -> [ ?QualifiedName, *PropertyDeclaration, *Binding, *CallbackConnection,
                      *CallbackDeclaration, *SubElement, *RepeatedElement, *PropertyAnimation,
                      *TwoWayBinding, *States, *Transitions, ?ChildrenPlaceholder ],
-        RepeatedElement -> [ ?DeclaredIdentifier, ?RepeatedIndex, Expression , Element],
+        RepeatedElement -> [ ?DeclaredIdentifier, ?RepeatedIndex, Expression , SubElement],
         RepeatedIndex -> [],
-        ConditionalElement -> [ Expression , Element],
-        CallbackDeclaration -> [ DeclaredIdentifier, *Type, ?ReturnType ],
+        ConditionalElement -> [ Expression , SubElement],
+        CallbackDeclaration -> [ DeclaredIdentifier, *Type, ?ReturnType, ?TwoWayBinding ],
         /// `-> type`  (but without the ->)
         ReturnType -> [Type],
         CallbackConnection -> [ *DeclaredIdentifier,  CodeBlock ],
         /// Declaration of a propery.
-        PropertyDeclaration-> [ Type , DeclaredIdentifier, ?BindingExpression, ?TwoWayBinding ],
+        PropertyDeclaration-> [ ?Type , DeclaredIdentifier, ?BindingExpression, ?TwoWayBinding ],
         /// QualifiedName are the properties name
         PropertyAnimation-> [ *QualifiedName, *Binding ],
         /// wraps Identifiers, like `Rectangle` or `SomeModule.SomeType`
@@ -455,7 +456,7 @@ mod parser_trait {
         type Checkpoint: Clone;
 
         /// Enter a new node.  The node is going to be finished when
-        /// The return value of this function is drop'ed
+        /// The return value of this function is dropped
         ///
         /// (do not re-implement this function, re-implement
         /// start_node_impl and finish_node_impl)
@@ -486,7 +487,7 @@ mod parser_trait {
         fn peek(&mut self) -> Token {
             self.nth(0)
         }
-        /// Peek the n'th token, not including whitespaces and comments
+        /// Peek the n'th token, not including whitespace and comments
         fn nth(&mut self, n: usize) -> Token;
         fn consume(&mut self);
         fn error(&mut self, e: impl Into<String>);
@@ -510,7 +511,7 @@ mod parser_trait {
             true
         }
 
-        /// consume everyting until reaching a token of this kind
+        /// consume everything until reaching a token of this kind
         fn until(&mut self, kind: SyntaxKind) {
             // FIXME! match {} () []
             while {
@@ -600,7 +601,7 @@ impl Parser for DefaultParser<'_> {
         self.builder.finish_node();
     }
 
-    /// Peek the n'th token, not including whitespaces and comments
+    /// Peek the n'th token, not including whitespace and comments
     fn nth(&mut self, mut n: usize) -> Token {
         self.consume_ws();
         let mut c = self.cursor;
