@@ -14,7 +14,7 @@ LICENSE END */
 //! which is not exposed. This passes create the viewport and fixes all property access
 //!
 //! It will also initialize proper geometry
-//! This pass must be called before the materialize_fake_properties as it going to be generare
+//! This pass must be called before the materialize_fake_properties as it going to be generate
 //! binding reference to fake properties
 
 use std::cell::RefCell;
@@ -58,19 +58,19 @@ fn create_viewport_element(flickable_elem: &ElementRc, native_rect: &Rc<NativeCl
         is_flickable_viewport: true,
         ..Element::default()
     }));
-    for (prop, ty) in &flickable.base_type.as_builtin().properties {
+    for (prop, info) in &flickable.base_type.as_builtin().properties {
         if let Some(vp_prop) = prop.strip_prefix("viewport_") {
             let nr = NamedReference::new(&viewport, vp_prop);
-            flickable.property_declarations.insert(prop.to_owned(), ty.clone().into());
+            flickable.property_declarations.insert(prop.to_owned(), info.ty.clone().into());
             match flickable.bindings.entry(prop.to_owned()) {
-                std::collections::hash_map::Entry::Occupied(entry) => {
+                std::collections::btree_map::Entry::Occupied(entry) => {
                     let entry = entry.into_mut();
                     entry.expression = Expression::TwoWayBinding(
                         nr,
                         Some(Box::new(std::mem::take(&mut entry.expression))),
                     )
                 }
-                std::collections::hash_map::Entry::Vacant(entry) => {
+                std::collections::btree_map::Entry::Vacant(entry) => {
                     entry.insert(Expression::TwoWayBinding(nr, None).into());
                 }
             }
@@ -92,16 +92,16 @@ fn fixup_geometry(flickable_elem: &ElementRc) {
                 .iter()
                 .filter(|x| is_layout(&x.borrow().base_type))
                 .map(|x| Expression::PropertyReference(NamedReference::new(x, prop)))
-                .fold1(|lhs, rhs| crate::expression_tree::min_max_expression(lhs, rhs, op))
+                .fold1(|lhs, rhs| crate::builtin_macros::min_max_expression(lhs, rhs, op))
         })
     };
 
     if !flickable_elem.borrow().bindings.contains_key("height") {
-        forward_minmax_of("maximum_height", '<');
+        forward_minmax_of("max_height", '<');
         forward_minmax_of("preferred_height", '<');
     }
     if !flickable_elem.borrow().bindings.contains_key("width") {
-        forward_minmax_of("maximum_width", '<');
+        forward_minmax_of("max_width", '<');
         forward_minmax_of("preferred_width", '<');
     }
     set_binding_if_not_explicit(flickable_elem, "viewport_width", || {
@@ -111,10 +111,10 @@ fn fixup_geometry(flickable_elem: &ElementRc) {
                 .children
                 .iter()
                 .filter(|x| is_layout(&x.borrow().base_type))
-                .map(|x| Expression::PropertyReference(NamedReference::new(x, "minimum_width")))
+                .map(|x| Expression::PropertyReference(NamedReference::new(x, "min_width")))
                 .fold(
                     Expression::PropertyReference(NamedReference::new(flickable_elem, "width")),
-                    |lhs, rhs| crate::expression_tree::min_max_expression(lhs, rhs, '>'),
+                    |lhs, rhs| crate::builtin_macros::min_max_expression(lhs, rhs, '>'),
                 ),
         )
     });
@@ -125,10 +125,10 @@ fn fixup_geometry(flickable_elem: &ElementRc) {
                 .children
                 .iter()
                 .filter(|x| is_layout(&x.borrow().base_type))
-                .map(|x| Expression::PropertyReference(NamedReference::new(x, "minimum_height")))
+                .map(|x| Expression::PropertyReference(NamedReference::new(x, "min_height")))
                 .fold(
                     Expression::PropertyReference(NamedReference::new(flickable_elem, "height")),
-                    |lhs, rhs| crate::expression_tree::min_max_expression(lhs, rhs, '>'),
+                    |lhs, rhs| crate::builtin_macros::min_max_expression(lhs, rhs, '>'),
                 ),
         )
     });
