@@ -60,8 +60,6 @@ fn lower_popup_window(
     };
 
     let parent_component = parent_element.borrow().enclosing_component.upgrade().unwrap();
-    // Because layout constraint which are supposed to be in the popup will not be lowered, the layout lowering should be done after
-    debug_assert!(parent_component.layouts.borrow().is_empty());
 
     // Remove the popup_window_element from its parent
     parent_element.borrow_mut().children.retain(|child| !Rc::ptr_eq(child, popup_window_element));
@@ -81,8 +79,8 @@ fn lower_popup_window(
 
     // Generate a x and y property, relative to the window coordinate
     // FIXME: this is a hack that doesn't always work, perhaps should we store an item ref or something
-    let coord_x = create_coodiate(&popup_comp, parent_stack, "x");
-    let coord_y = create_coodiate(&popup_comp, parent_stack, "y");
+    let coord_x = create_coordinate(&popup_comp, parent_stack, "x");
+    let coord_y = create_coordinate(&popup_comp, parent_stack, "y");
 
     // Throw error when accessing the popup from outside
     // FIXME:
@@ -105,7 +103,7 @@ fn lower_popup_window(
     });
 }
 
-fn create_coodiate(
+fn create_coordinate(
     popup_comp: &Rc<Component>,
     parent_stack: &[ElementRc],
     coord: &str,
@@ -119,14 +117,6 @@ fn create_coodiate(
         .unwrap_or(Expression::NumberLiteral(0., crate::expression_tree::Unit::Phx));
 
     for parent in parent_stack {
-        match &parent.borrow().base_type {
-            Type::Builtin(b) => {
-                if !b.properties.contains_key(coord) {
-                    continue;
-                }
-            }
-            _ => continue,
-        }
         expression = Expression::BinaryExpression {
             lhs: Box::new(expression),
             rhs: Box::new(Expression::PropertyReference(NamedReference::new(parent, coord))),
