@@ -12,7 +12,7 @@ LICENSE END */
 //! Before this pass, all the expression are of type Expression::Uncompiled,
 //! and there should no longer be Uncompiled expression after this pass.
 //!
-//! Most of the code for the resolving actualy lies in the expression_tree module
+//! Most of the code for the resolving actually lies in the expression_tree module
 
 use crate::diagnostics::{BuildDiagnostics, Spanned};
 use crate::expression_tree::*;
@@ -21,10 +21,10 @@ use crate::lookup::{LookupCtx, LookupObject};
 use crate::object_tree::*;
 use crate::parser::{identifier_text, syntax_nodes, NodeOrToken, SyntaxKind, SyntaxNode};
 use crate::typeregister::TypeRegister;
+use std::collections::HashMap;
+use std::rc::Rc;
 
-use std::{collections::HashMap, rc::Rc};
-
-/// This represeresent a scope for the Component, where Component is the repeated component, but
+/// This represents a scope for the Component, where Component is the repeated component, but
 /// does not represent a component in the .60 file
 #[derive(Clone)]
 struct ComponentScope(Vec<ElementRc>);
@@ -52,7 +52,7 @@ fn resolve_expression(
 
         let new_expr = match node.kind() {
             SyntaxKind::CallbackConnection => {
-                //FIXME: proper callback suport (node is a codeblock)
+                //FIXME: proper callback support (node is a codeblock)
                 Expression::from_callback_connection(node.clone().into(), &mut lookup_ctx)
             }
             SyntaxKind::Expression => {
@@ -64,6 +64,11 @@ fn resolve_expression(
                 Expression::from_binding_expression_node(node.clone(), &mut lookup_ctx)
             }
             SyntaxKind::TwoWayBinding => {
+                if lookup_ctx.property_type == Type::Invalid {
+                    // An attempt to resolve this already failed when trying to resolve the property type
+                    assert!(diag.has_error());
+                    return;
+                }
                 Expression::from_two_way_binding(node.clone().into(), &mut lookup_ctx)
             }
             _ => {
@@ -134,7 +139,7 @@ impl Expression {
             })
             .unwrap_or(Self::Invalid);
         if ctx.property_type == Type::LogicalLength && e.ty() == Type::Percent {
-            // See if a conversion from percentage to lenght is allowed
+            // See if a conversion from percentage to length is allowed
             const RELATIVE_TO_PARENT_PROPERTIES: [&str; 2] = ["width", "height"];
             let property_name = ctx.property_name.unwrap_or_default();
             if RELATIVE_TO_PARENT_PROPERTIES.contains(&property_name) {
@@ -142,7 +147,7 @@ impl Expression {
             } else {
                 ctx.diag.push_error(
                     format!(
-                        "Automatic conversion from percentage to lenght is only possible for the properties {}",
+                        "Automatic conversion from percentage to length is only possible for the properties {}",
                         RELATIVE_TO_PARENT_PROPERTIES.join(" and ")
                     ),
                     &node
@@ -219,12 +224,15 @@ impl Expression {
         )
     }
 
-    fn from_two_way_binding(node: syntax_nodes::TwoWayBinding, ctx: &mut LookupCtx) -> Expression {
+    pub fn from_two_way_binding(
+        node: syntax_nodes::TwoWayBinding,
+        ctx: &mut LookupCtx,
+    ) -> Expression {
         let e = Self::from_expression_node(node.Expression(), ctx);
         let ty = e.ty();
         match e {
             Expression::PropertyReference(n) => {
-                if ty != ctx.property_type {
+                if ty != ctx.property_type && ctx.property_type != Type::InferredProperty {
                     ctx.diag.push_error(
                         "The property does not have the same type as the bound property".into(),
                         &node,
@@ -232,12 +240,20 @@ impl Expression {
                 }
                 Expression::TwoWayBinding(n, None)
             }
+            Expression::CallbackReference(n) => {
+                if ctx.property_type != Type::InferredCallback && ty != ctx.property_type {
+                    ctx.diag.push_error("Cannot bind to a callback".into(), &node);
+                    Expression::Invalid
+                } else {
+                    Expression::TwoWayBinding(n, None)
+                }
+            }
             _ => {
                 ctx.diag.push_error(
                     "The expression in a two way binding must be a property reference".into(),
                     &node,
                 );
-                e
+                Expression::Invalid
             }
         }
     }
@@ -283,7 +299,7 @@ impl Expression {
             .or_else(|| {
                 node.FunctionCallExpression().map(|n| Self::from_function_call_node(n, ctx))
             })
-            .or_else(|| node.SelfAssignment().map(|n| Self::from_self_assignement_node(n, ctx)))
+            .or_else(|| node.SelfAssignment().map(|n| Self::from_self_assignment_node(n, ctx)))
             .or_else(|| node.BinaryExpression().map(|n| Self::from_binary_expression_node(n, ctx)))
             .or_else(|| {
                 node.UnaryOpExpression().map(|n| Self::from_unaryop_expression_node(n, ctx))
@@ -384,7 +400,7 @@ impl Expression {
                     )),
                 }
             } else {
-                // To faciliate color literal conversion, adjust the expected return type.
+                // To facilitate color literal conversion, adjust the expected return type.
                 let e = {
                     let old_property_type = std::mem::replace(&mut ctx.property_type, Type::Color);
                     let e =
@@ -423,7 +439,7 @@ impl Expression {
                 rest.iter().position(|s| !matches!(s.1, Expression::Invalid)).unwrap_or(rest.len());
             if pos > 0 {
                 let (middle, after) = rest.split_at_mut(pos);
-                let begin = &before.last().expect("The first should never be invlid").1;
+                let begin = &before.last().expect("The first should never be invalid").1;
                 let end = &after.last().expect("The last should never be invalid").1;
                 for (i, (_, e)) in middle.iter_mut().enumerate() {
                     debug_assert!(matches!(e, Expression::Invalid));
@@ -479,7 +495,7 @@ impl Expression {
                     // Attempt to recover if the user wanted to write "-"
                     let first_str = &first.text()[0..minus_pos];
                     if global_lookup.lookup(ctx, first_str).is_some() {
-                        ctx.diag.push_error(format!("Unknown unqualified identifier '{}'. Use space before the '-' if you meant a substraction.", first.text()), &node);
+                        ctx.diag.push_error(format!("Unknown unqualified identifier '{}'. Use space before the '-' if you meant a subtraction", first.text()), &node);
                         return Expression::Invalid;
                     }
                 }
@@ -547,56 +563,25 @@ impl Expression {
         node: syntax_nodes::FunctionCallExpression,
         ctx: &mut LookupCtx,
     ) -> Expression {
-        let mut sub_expr = node
-            .Expression()
-            .map(|n| (Self::from_expression_node(n.clone(), ctx), NodeOrToken::from((*n).clone())));
+        let mut sub_expr = node.Expression().map(|n| {
+            (Self::from_expression_node(n.clone(), ctx), Some(NodeOrToken::from((*n).clone())))
+        });
 
         let mut arguments = Vec::new();
 
-        let (function, f_node) =
-            sub_expr.next().unwrap_or_else(|| (Expression::Invalid, (*node).clone().into()));
+        let (function, _) =
+            sub_expr.next().unwrap_or_else(|| (Expression::Invalid, Some((*node).clone().into())));
 
         let function = match function {
-            Expression::BuiltinMacroReference(mac, n) => match mac {
-                BuiltinMacroFunction::Min => {
-                    return min_max_macro(n, '<', sub_expr.collect(), &mut ctx.diag);
-                }
-                BuiltinMacroFunction::Max => {
-                    return min_max_macro(n, '>', sub_expr.collect(), &mut ctx.diag);
-                }
-                BuiltinMacroFunction::CubicBezier => {
-                    let mut has_error = None;
-                    // FIXME: this is not pretty to be handling there.
-                    // Maybe "cubic_bezier" should be a function that is lowered later
-                    let mut a = || match sub_expr.next() {
-                        None => {
-                            has_error.get_or_insert((f_node.clone(), "Not enough arguments"));
-                            0.
-                        }
-                        Some((Expression::NumberLiteral(val, Unit::None), _)) => val as f32,
-                        Some((_, n)) => {
-                            has_error.get_or_insert((
-                                n,
-                                "Arguments to cubic bezier curve must be number literal",
-                            ));
-                            0.
-                        }
-                    };
-                    let expr =
-                        Expression::EasingCurve(EasingCurve::CubicBezier(a(), a(), a(), a()));
-                    if let Some((_, n)) = sub_expr.next() {
-                        has_error.get_or_insert((n, "Too many argument for bezier curve"));
-                    }
-                    if let Some((n, msg)) = has_error {
-                        ctx.diag.push_error(msg.into(), &n);
-                    }
-
-                    return expr;
-                }
-                BuiltinMacroFunction::Rgb => {
-                    return rgb_macro(n, sub_expr.collect(), &mut ctx.diag);
-                }
-            },
+            Expression::BuiltinMacroReference(mac, n) => {
+                arguments.extend(sub_expr);
+                return crate::builtin_macros::lower_macro(
+                    mac,
+                    n,
+                    arguments.into_iter(),
+                    &mut ctx.diag,
+                );
+            }
             Expression::MemberFunction { base, base_node, member } => {
                 arguments.push((*base, base_node));
                 member
@@ -638,12 +623,12 @@ impl Expression {
         }
     }
 
-    fn from_self_assignement_node(
+    fn from_self_assignment_node(
         node: syntax_nodes::SelfAssignment,
         ctx: &mut LookupCtx,
     ) -> Expression {
         let (lhs_n, rhs_n) = node.Expression();
-        let lhs = Self::from_expression_node(lhs_n, ctx);
+        let mut lhs = Self::from_expression_node(lhs_n, ctx);
         let op = None
             .or(node.child_token(SyntaxKind::PlusEqual).and(Some('+')))
             .or(node.child_token(SyntaxKind::MinusEqual).and(Some('-')))
@@ -651,11 +636,11 @@ impl Expression {
             .or(node.child_token(SyntaxKind::DivEqual).and(Some('/')))
             .or(node.child_token(SyntaxKind::Equal).and(Some('=')))
             .unwrap_or('_');
-        if !lhs.is_rw() && lhs.ty() != Type::Invalid {
+        if !lhs.try_set_rw() && lhs.ty() != Type::Invalid {
             ctx.diag.push_error(
                 format!(
-                    "{} need to be done on a property",
-                    if op == '=' { "Assignement" } else { "Self assignement" }
+                    "{} needs to be done on a property",
+                    if op == '=' { "Assignment" } else { "Self assignment" }
                 ),
                 &node,
             );
@@ -925,77 +910,6 @@ impl Expression {
     }
 }
 
-fn min_max_macro(
-    node: Option<NodeOrToken>,
-    op: char,
-    args: Vec<(Expression, NodeOrToken)>,
-    diag: &mut BuildDiagnostics,
-) -> Expression {
-    if args.is_empty() {
-        diag.push_error("Needs at least one argument".into(), &node);
-        return Expression::Invalid;
-    }
-    let mut args = args.into_iter();
-    let (mut base, arg_node) = args.next().unwrap();
-    let ty = match base.ty() {
-        Type::Float32 => Type::Float32,
-        // In case there are other floats, we don't want to conver tthe result to int
-        Type::Int32 => Type::Float32,
-        Type::PhysicalLength => Type::PhysicalLength,
-        Type::LogicalLength => Type::LogicalLength,
-        Type::Duration => Type::Duration,
-        Type::Angle => Type::Angle,
-        Type::Percent => Type::Float32,
-        _ => {
-            diag.push_error("Invalid argument type".into(), &arg_node);
-            return Expression::Invalid;
-        }
-    };
-    for (next, arg_node) in args {
-        let rhs = next.maybe_convert_to(ty.clone(), &arg_node, diag);
-        base = crate::expression_tree::min_max_expression(base, rhs, op);
-    }
-    base
-}
-
-fn rgb_macro(
-    node: Option<NodeOrToken>,
-    args: Vec<(Expression, NodeOrToken)>,
-    diag: &mut BuildDiagnostics,
-) -> Expression {
-    if args.len() < 3 {
-        diag.push_error("Needs 3 or 4 argument".into(), &node);
-        return Expression::Invalid;
-    }
-    let mut arguments: Vec<_> = args
-        .into_iter()
-        .enumerate()
-        .map(|(i, (expr, n))| {
-            if i < 3 {
-                if expr.ty() == Type::Percent {
-                    Expression::BinaryExpression {
-                        lhs: Box::new(expr.maybe_convert_to(Type::Float32, &n, diag)),
-                        rhs: Box::new(Expression::NumberLiteral(255., Unit::None)),
-                        op: '*',
-                    }
-                } else {
-                    expr.maybe_convert_to(Type::Int32, &n, diag)
-                }
-            } else {
-                expr.maybe_convert_to(Type::Float32, &n, diag)
-            }
-        })
-        .collect();
-    if arguments.len() < 4 {
-        arguments.push(Expression::NumberLiteral(1., Unit::None))
-    }
-    Expression::FunctionCall {
-        function: Box::new(Expression::BuiltinFunctionReference(BuiltinFunction::Rgb)),
-        arguments,
-        source_location: Some(node.to_source_location()),
-    }
-}
-
 fn continue_lookup_within_element(
     elem: &ElementRc,
     it: &mut impl Iterator<Item = crate::parser::SyntaxToken>,
@@ -1029,7 +943,7 @@ fn continue_lookup_within_element(
         let member = elem.borrow().base_type.lookup_member_function(&resolved_name);
         Expression::MemberFunction {
             base: Box::new(Expression::ElementReference(Rc::downgrade(elem))),
-            base_node: node.into(),
+            base_node: Some(node.into()),
             member: Box::new(member),
         }
     } else {
@@ -1042,7 +956,10 @@ fn continue_lookup_within_element(
                 }
                 Type::Component(c) => format!("Element '{}'", c.id),
                 Type::Builtin(b) => format!("Element '{}'", b.name),
-                _ => unreachable!(),
+                _ => {
+                    assert!(ctx.diag.has_error());
+                    return;
+                }
             };
             ctx.diag.push_error(
                 format!("{} does not have a property '{}'.{}", what, second.text(), extra),
@@ -1054,7 +971,7 @@ fn continue_lookup_within_element(
             if elem.borrow().lookup_property(&second.text()[0..minus_pos]).property_type
                 != Type::Invalid
             {
-                err(" Use space before the '-' if you meant a substraction.");
+                err(" Use space before the '-' if you meant a subtraction");
                 return Expression::Invalid;
             }
         }
@@ -1068,86 +985,28 @@ fn maybe_lookup_object(
     it: impl Iterator<Item = crate::parser::SyntaxToken>,
     ctx: &mut LookupCtx,
 ) -> Expression {
-    fn error_or_try_minus(
-        ctx: &mut LookupCtx,
-        ident: crate::parser::SyntaxToken,
-        lookup: impl Fn(&str) -> bool,
-    ) -> Expression {
-        if let Some(minus_pos) = ident.text().find('-') {
-            if lookup(&ident.text()[0..minus_pos]) {
-                ctx.diag.push_error(format!("Cannot access the field '{}'. Use space before the '-' if you meant a substraction.", ident.text()), &ident);
-                return Expression::Invalid;
-            }
-        }
-        ctx.diag.push_error(format!("Cannot access the field '{}'", ident.text()), &ident);
-        Expression::Invalid
-    }
-
     for next in it {
         let next_str = crate::parser::normalize_identifier(next.text());
-        match base.ty() {
-            Type::Struct { fields, .. } => {
-                if fields.get(next_str.as_str()).is_some() {
-                    base = Expression::StructFieldAccess {
-                        base: Box::new(std::mem::replace(&mut base, Expression::Invalid)),
-                        name: next_str,
+        ctx.current_token = Some(next.clone().into());
+        match base.lookup(ctx, &next_str) {
+            Some(result) => {
+                base = result.expression;
+            }
+            None => {
+                if let Some(minus_pos) = next.text().find('-') {
+                    if base.lookup(ctx, &next.text()[0..minus_pos]).is_some() {
+                        ctx.diag.push_error(format!("Cannot access the field '{}'. Use space before the '-' if you meant a subtraction", next.text()), &next);
+                        return Expression::Invalid;
                     }
-                } else {
-                    return error_or_try_minus(ctx, next, |x| fields.get(x).is_some());
                 }
-            }
-            Type::Component(c) => {
-                let PropertyLookupResult { resolved_name, property_type } =
-                    c.root_element.borrow().lookup_property(next_str.as_str());
-                if property_type != Type::Invalid {
-                    base = Expression::StructFieldAccess {
-                        base: Box::new(std::mem::replace(&mut base, Expression::Invalid)),
-                        name: resolved_name.to_string(),
-                    }
-                } else {
-                    return error_or_try_minus(ctx, next, |x| {
-                        c.root_element.borrow().lookup_property(x).property_type != Type::Invalid
-                    });
-                }
-            }
-            Type::String => {
-                return Expression::MemberFunction {
-                    base: Box::new(base),
-                    base_node: next.clone().into(), // Note that this is not the base_node, but the function's node
-                    member: Box::new(match next_str.as_str() {
-                        "is_float" => {
-                            Expression::BuiltinFunctionReference(BuiltinFunction::StringIsFloat)
-                        }
-                        "to_float" => {
-                            Expression::BuiltinFunctionReference(BuiltinFunction::StringToFloat)
-                        }
-                        _ => {
-                            ctx.diag.push_error("Cannot access fields of string".into(), &next);
-                            return Expression::Invalid;
-                        }
-                    }),
+                let ty_descr = match base.ty() {
+                    Type::Struct { .. } => String::new(),
+                    ty => format!(" of {}", ty),
                 };
-            }
-            Type::Color => {
-                return Expression::MemberFunction {
-                    base: Box::new(base),
-                    base_node: next.clone().into(), // Note that this is not the base_node, but the function's node
-                    member: Box::new(match next_str.as_str() {
-                        "brighter" => {
-                            Expression::BuiltinFunctionReference(BuiltinFunction::ColorBrighter)
-                        }
-                        "darker" => {
-                            Expression::BuiltinFunctionReference(BuiltinFunction::ColorDarker)
-                        }
-                        _ => {
-                            ctx.diag.push_error("Cannot access fields of color".into(), &next);
-                            return Expression::Invalid;
-                        }
-                    }),
-                };
-            }
-            _ => {
-                ctx.diag.push_error("Cannot access fields of property".into(), &next);
+                ctx.diag.push_error(
+                    format!("Cannot access the field '{}'{}", next.text(), ty_descr),
+                    &next,
+                );
                 return Expression::Invalid;
             }
         }
