@@ -66,11 +66,11 @@ fn process_diagnostics(
 ) -> std::io::Result<bool> {
     let mut success = true;
 
-    let path = cannonical(path);
+    let path = canonical(path);
 
     let mut diags = compile_diagnostics
         .iter()
-        .filter(|d| cannonical(d.source_file().unwrap()) == path)
+        .filter(|d| canonical(d.source_file().unwrap()) == path)
         .collect::<Vec<_>>();
 
     let lines = source
@@ -131,7 +131,7 @@ fn process_diagnostics(
         }
     }
     if !diags.is_empty() {
-        println!("{:?}: Unexptected errors/warnings: {:#?}", path, diags);
+        println!("{:?}: Unexpected errors/warnings: {:#?}", path, diags);
 
         #[cfg(feature = "display-diagnostics")]
         if !silent {
@@ -147,7 +147,7 @@ fn process_diagnostics(
     Ok(success)
 }
 
-fn cannonical(path: &Path) -> PathBuf {
+fn canonical(path: &Path) -> PathBuf {
     path.canonicalize().unwrap_or_else(|_| path.to_owned())
 }
 
