@@ -12,7 +12,7 @@ use std::collections::{HashMap, HashSet};
 use std::rc::Rc;
 
 use crate::expression_tree::{BuiltinFunction, Expression};
-use crate::langtype::{Enumeration, Type};
+use crate::langtype::{BuiltinPropertyInfo, Enumeration, PropertyLookupResult, Type};
 use crate::object_tree::Component;
 
 pub(crate) const RESERVED_GEOMETRY_PROPERTIES: &'static [(&'static str, Type)] = &[
@@ -20,13 +20,14 @@ pub(crate) const RESERVED_GEOMETRY_PROPERTIES: &'static [(&'static str, Type)] =
     ("y", Type::LogicalLength),
     ("width", Type::LogicalLength),
     ("height", Type::LogicalLength),
+    ("z", Type::Float32),
 ];
 
 const RESERVED_LAYOUT_PROPERTIES: &'static [(&'static str, Type)] = &[
-    ("minimum_width", Type::LogicalLength),
-    ("minimum_height", Type::LogicalLength),
-    ("maximum_width", Type::LogicalLength),
-    ("maximum_height", Type::LogicalLength),
+    ("min_width", Type::LogicalLength),
+    ("min_height", Type::LogicalLength),
+    ("max_width", Type::LogicalLength),
+    ("max_height", Type::LogicalLength),
     ("padding", Type::LogicalLength),
     ("padding_left", Type::LogicalLength),
     ("padding_right", Type::LogicalLength),
@@ -40,7 +41,6 @@ const RESERVED_LAYOUT_PROPERTIES: &'static [(&'static str, Type)] = &[
     ("row", Type::Int32),
     ("colspan", Type::Int32),
     ("rowspan", Type::Int32),
-    ("clip", Type::Bool),
 ];
 
 const RESERVED_OTHER_PROPERTIES: &'static [(&'static str, Type)] = &[
@@ -56,32 +56,50 @@ pub(crate) const RESERVED_DROP_SHADOW_PROPERTIES: &'static [(&'static str, Type)
     ("drop_shadow_color", Type::Color),
 ];
 
-/// reserved property injected in every item
-pub fn reserved_property(name: &str) -> Type {
-    for (p, t) in RESERVED_GEOMETRY_PROPERTIES
+/// list of reserved property injected in every item
+pub fn reserved_properties() -> impl Iterator<Item = (&'static str, Type)> {
+    RESERVED_GEOMETRY_PROPERTIES
         .iter()
         .chain(RESERVED_LAYOUT_PROPERTIES.iter())
         .chain(RESERVED_OTHER_PROPERTIES.iter())
         .chain(RESERVED_DROP_SHADOW_PROPERTIES.iter())
-        .chain(
-            [
-                ("forward_focus", Type::ElementReference),
-                ("focus", BuiltinFunction::SetFocusItem.ty()),
-            ]
-            .iter(),
-        )
-    {
-        if *p == name {
-            return t.clone();
+        .map(|(k, v)| (*k, v.clone()))
+        .chain(std::array::IntoIter::new([
+            ("forward_focus", Type::ElementReference),
+            ("focus", BuiltinFunction::SetFocusItem.ty()),
+        ]))
+}
+
+/// lookup reserved property injected in every item
+pub fn reserved_property(name: &str) -> PropertyLookupResult {
+    for (p, t) in reserved_properties() {
+        if p == name {
+            return PropertyLookupResult { property_type: t, resolved_name: name.into() };
         }
     }
-    Type::Invalid
+
+    // Report deprecated known reserved properties (maximum_width, minimum_height, ...)
+    for pre in &["min", "max"] {
+        if let Some(a) = name.strip_prefix(pre) {
+            for suf in &["width", "height"] {
+                if let Some(b) = a.strip_suffix(suf) {
+                    if b == "imum_" {
+                        return PropertyLookupResult {
+                            property_type: Type::LogicalLength,
+                            resolved_name: format!("{}_{}", pre, suf).into(),
+                        };
+                    }
+                }
+            }
+        }
+    }
+    PropertyLookupResult { resolved_name: name.into(), property_type: Type::Invalid }
 }
 
 /// These member functions are injected in every time
 pub fn reserved_member_function(name: &str) -> Expression {
     for (m, e) in [
-        ("focus", Expression::BuiltinFunctionReference(BuiltinFunction::SetFocusItem)), // match for callable "focus" property
+        ("focus", Expression::BuiltinFunctionReference(BuiltinFunction::SetFocusItem, None)), // match for callable "focus" property
     ]
     .iter()
     {
@@ -172,13 +190,13 @@ impl TypeRegister {
 
         match &mut register.types.get_mut("PopupWindow").unwrap() {
             Type::Builtin(ref mut b) => {
-                Rc::get_mut(b)
-                    .unwrap()
-                    .properties
-                    .insert("show".into(), BuiltinFunction::ShowPopupWindow.ty());
+                Rc::get_mut(b).unwrap().properties.insert(
+                    "show".into(),
+                    BuiltinPropertyInfo::new(BuiltinFunction::ShowPopupWindow.ty()),
+                );
                 Rc::get_mut(b).unwrap().member_functions.insert(
                     "show".into(),
-                    Expression::BuiltinFunctionReference(BuiltinFunction::ShowPopupWindow),
+                    Expression::BuiltinFunctionReference(BuiltinFunction::ShowPopupWindow, None),
                 );
             }
             _ => unreachable!(),
